/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.common.comms;

import org.waveprotocol.wave.federation.ProtocolDocumentOperation;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for DocumentSnapshot.
 *
 * Generated from org/waveprotocol/box/common/comms/waveclient-rpc.proto. Do not edit.
 */
public interface DocumentSnapshot {

  /** Does a deep copy from model. */
  void copyFrom(DocumentSnapshot model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns documentId, or null if hasn't been set. */
  String getDocumentId();

  /** Sets documentId. */
  void setDocumentId(String documentId);

  /** Returns documentOperation, or null if hasn't been set. */
  ProtocolDocumentOperation getDocumentOperation();

  /** Sets documentOperation. */
  void setDocumentOperation(ProtocolDocumentOperation documentOperation);

  /** Returns author, or null if hasn't been set. */
  String getAuthor();

  /** Sets author. */
  void setAuthor(String author);

  /** Returns contributor, or null if hasn't been set. */
  List<String> getContributor();

  /** Adds an element to contributor. */
  void addContributor(String value);

  /** Adds a list of elements to contributor. */
  void addAllContributor(List<String> contributor);

  /** Returns the nth element of contributor. */
  String getContributor(int n);

  /** Sets the nth element of contributor. */
  void setContributor(int n, String value);

  /** Returns the length of contributor. */
  int getContributorSize();

  /** Clears contributor. */
  void clearContributor();

  /** Returns lastModifiedVersion, or null if hasn't been set. */
  long getLastModifiedVersion();

  /** Sets lastModifiedVersion. */
  void setLastModifiedVersion(long lastModifiedVersion);

  /** Returns lastModifiedTime, or null if hasn't been set. */
  long getLastModifiedTime();

  /** Sets lastModifiedTime. */
  void setLastModifiedTime(long lastModifiedTime);
}