/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.common.comms;

import org.waveprotocol.box.common.comms.WaveletVersion;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for ProtocolOpenRequest.
 *
 * Generated from org/waveprotocol/box/common/comms/waveclient-rpc.proto. Do not edit.
 */
public interface ProtocolOpenRequest {

  /** Does a deep copy from model. */
  void copyFrom(ProtocolOpenRequest model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns participantId, or null if hasn't been set. */
  String getParticipantId();

  /** Sets participantId. */
  void setParticipantId(String participantId);

  /** Returns waveId, or null if hasn't been set. */
  String getWaveId();

  /** Sets waveId. */
  void setWaveId(String waveId);

  /** Returns waveletIdPrefix, or null if hasn't been set. */
  List<String> getWaveletIdPrefix();

  /** Adds an element to waveletIdPrefix. */
  void addWaveletIdPrefix(String value);

  /** Adds a list of elements to waveletIdPrefix. */
  void addAllWaveletIdPrefix(List<String> waveletIdPrefix);

  /** Returns the nth element of waveletIdPrefix. */
  String getWaveletIdPrefix(int n);

  /** Sets the nth element of waveletIdPrefix. */
  void setWaveletIdPrefix(int n, String value);

  /** Returns the length of waveletIdPrefix. */
  int getWaveletIdPrefixSize();

  /** Clears waveletIdPrefix. */
  void clearWaveletIdPrefix();

  /** Returns knownWavelet, or null if hasn't been set. */
  List<? extends WaveletVersion> getKnownWavelet();

  /** Adds an element to knownWavelet. */
  void addKnownWavelet(WaveletVersion value);

  /** Adds a list of elements to knownWavelet. */
  void addAllKnownWavelet(List<? extends WaveletVersion> knownWavelet);

  /** Returns the nth element of knownWavelet. */
  WaveletVersion getKnownWavelet(int n);

  /** Sets the nth element of knownWavelet. */
  void setKnownWavelet(int n, WaveletVersion value);

  /** Returns the length of knownWavelet. */
  int getKnownWaveletSize();

  /** Clears knownWavelet. */
  void clearKnownWavelet();
}