/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.common.comms;

import org.waveprotocol.box.common.comms.ProtocolSubmitRequest.*;
import org.waveprotocol.wave.federation.ProtocolWaveletDelta;
import org.waveprotocol.wave.federation.ProtocolWaveletDeltaUtil;
import org.waveprotocol.wave.communication.Blob;
import java.util.Iterator;
import java.util.List;

/**
 * Compares {@link ProtocolSubmitRequest}s for equality.
 *
 * Generated from org/waveprotocol/box/common/comms/waveclient-rpc.proto. Do not edit.
 */
public final class ProtocolSubmitRequestUtil {
  private ProtocolSubmitRequestUtil() {
  }

  /** Returns true if m1 and m2 are structurally equal. */
  public static boolean isEqual(ProtocolSubmitRequest m1, ProtocolSubmitRequest m2) {
    if (!m1.getWaveletName().equals(m2.getWaveletName())) return false;
    if (!ProtocolWaveletDeltaUtil.isEqual(m1.getDelta(), m2.getDelta())) return false;
    if (m1.hasChannelId() != m2.hasChannelId()) return false;
    if (m1.hasChannelId() && !m1.getChannelId().equals(m2.getChannelId())) return false;
    return true;
  }

  /** Returns true if m1 and m2 are equal according to isEqual. */
  public static boolean areAllEqual(List<? extends ProtocolSubmitRequest> m1,
  List<? extends ProtocolSubmitRequest> m2) {
    if (m1.size() != m2.size()) return false;
    Iterator<? extends ProtocolSubmitRequest> i1 = m1.iterator();
    Iterator<? extends ProtocolSubmitRequest> i2 = m2.iterator();
    while (i1.hasNext()) {
      if (!isEqual(i1.next(), i2.next())) return false;
    }
    return true;
  }

  /** Returns a structural hash code of message. */
  public static int getHashCode(ProtocolSubmitRequest message) {
    int result = 1;
    result = (31 * result) + message.getWaveletName().hashCode();
    result = (31 * result) + message.getDelta().hashCode();
    result = (31 * result) + (message.hasChannelId() ? message.getChannelId().hashCode() : 0);
    return result;
  }

}