/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.common.comms;

import org.waveprotocol.box.common.comms.ProtocolSubmitResponse.*;
import org.waveprotocol.wave.federation.ProtocolHashedVersion;
import org.waveprotocol.wave.federation.ProtocolHashedVersionUtil;
import org.waveprotocol.wave.communication.Blob;
import java.util.Iterator;
import java.util.List;

/**
 * Compares {@link ProtocolSubmitResponse}s for equality.
 *
 * Generated from org/waveprotocol/box/common/comms/waveclient-rpc.proto. Do not edit.
 */
public final class ProtocolSubmitResponseUtil {
  private ProtocolSubmitResponseUtil() {
  }

  /** Returns true if m1 and m2 are structurally equal. */
  public static boolean isEqual(ProtocolSubmitResponse m1, ProtocolSubmitResponse m2) {
    if (m1.getOperationsApplied() != m2.getOperationsApplied()) return false;
    if (m1.hasErrorMessage() != m2.hasErrorMessage()) return false;
    if (m1.hasErrorMessage() && !m1.getErrorMessage().equals(m2.getErrorMessage())) return false;
    if (m1.hasHashedVersionAfterApplication() != m2.hasHashedVersionAfterApplication()) return false;
    if (m1.hasHashedVersionAfterApplication() && !ProtocolHashedVersionUtil.isEqual(m1.getHashedVersionAfterApplication(), m2.getHashedVersionAfterApplication())) return false;
    return true;
  }

  /** Returns true if m1 and m2 are equal according to isEqual. */
  public static boolean areAllEqual(List<? extends ProtocolSubmitResponse> m1,
  List<? extends ProtocolSubmitResponse> m2) {
    if (m1.size() != m2.size()) return false;
    Iterator<? extends ProtocolSubmitResponse> i1 = m1.iterator();
    Iterator<? extends ProtocolSubmitResponse> i2 = m2.iterator();
    while (i1.hasNext()) {
      if (!isEqual(i1.next(), i2.next())) return false;
    }
    return true;
  }

  /** Returns a structural hash code of message. */
  public static int getHashCode(ProtocolSubmitResponse message) {
    int result = 1;
    result = (31 * result) + Integer.valueOf(message.getOperationsApplied()).hashCode();
    result = (31 * result) + (message.hasErrorMessage() ? message.getErrorMessage().hashCode() : 0);
    result = (31 * result) + (message.hasHashedVersionAfterApplication() ? message.getHashedVersionAfterApplication().hashCode() : 0);
    return result;
  }

}