/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.common.comms;

import org.waveprotocol.wave.federation.ProtocolWaveletDelta;
import org.waveprotocol.wave.federation.ProtocolHashedVersion;
import org.waveprotocol.box.common.comms.WaveletSnapshot;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for ProtocolWaveletUpdate.
 *
 * Generated from org/waveprotocol/box/common/comms/waveclient-rpc.proto. Do not edit.
 */
public interface ProtocolWaveletUpdate {

  /** Does a deep copy from model. */
  void copyFrom(ProtocolWaveletUpdate model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns waveletName, or null if hasn't been set. */
  String getWaveletName();

  /** Sets waveletName. */
  void setWaveletName(String waveletName);

  /** Returns appliedDelta, or null if hasn't been set. */
  List<? extends ProtocolWaveletDelta> getAppliedDelta();

  /** Adds an element to appliedDelta. */
  void addAppliedDelta(ProtocolWaveletDelta value);

  /** Adds a list of elements to appliedDelta. */
  void addAllAppliedDelta(List<? extends ProtocolWaveletDelta> appliedDelta);

  /** Returns the nth element of appliedDelta. */
  ProtocolWaveletDelta getAppliedDelta(int n);

  /** Sets the nth element of appliedDelta. */
  void setAppliedDelta(int n, ProtocolWaveletDelta value);

  /** Returns the length of appliedDelta. */
  int getAppliedDeltaSize();

  /** Clears appliedDelta. */
  void clearAppliedDelta();

  /** Returns whether commitNotice has been set. */
  boolean hasCommitNotice();

  /** Clears the value of commitNotice. */
  void clearCommitNotice();

  /** Returns commitNotice, or null if hasn't been set. */
  ProtocolHashedVersion getCommitNotice();

  /** Sets commitNotice. */
  void setCommitNotice(ProtocolHashedVersion commitNotice);

  /** Returns whether resultingVersion has been set. */
  boolean hasResultingVersion();

  /** Clears the value of resultingVersion. */
  void clearResultingVersion();

  /** Returns resultingVersion, or null if hasn't been set. */
  ProtocolHashedVersion getResultingVersion();

  /** Sets resultingVersion. */
  void setResultingVersion(ProtocolHashedVersion resultingVersion);

  /** Returns whether snapshot has been set. */
  boolean hasSnapshot();

  /** Clears the value of snapshot. */
  void clearSnapshot();

  /** Returns snapshot, or null if hasn't been set. */
  WaveletSnapshot getSnapshot();

  /** Sets snapshot. */
  void setSnapshot(WaveletSnapshot snapshot);

  /** Returns whether marker has been set. */
  boolean hasMarker();

  /** Clears the value of marker. */
  void clearMarker();

  /** Returns marker, or null if hasn't been set. */
  boolean getMarker();

  /** Sets marker. */
  void setMarker(boolean marker);

  /** Returns whether channelId has been set. */
  boolean hasChannelId();

  /** Clears the value of channelId. */
  void clearChannelId();

  /** Returns channelId, or null if hasn't been set. */
  String getChannelId();

  /** Sets channelId. */
  void setChannelId(String channelId);
}