/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.common.comms;

import org.waveprotocol.box.common.comms.ProtocolWaveletUpdate.*;
import org.waveprotocol.wave.federation.ProtocolWaveletDelta;
import org.waveprotocol.wave.federation.ProtocolHashedVersion;
import org.waveprotocol.box.common.comms.WaveletSnapshot;
import org.waveprotocol.wave.federation.ProtocolWaveletDeltaUtil;
import org.waveprotocol.wave.federation.ProtocolHashedVersionUtil;
import org.waveprotocol.box.common.comms.WaveletSnapshotUtil;
import org.waveprotocol.wave.communication.Blob;
import java.util.Iterator;
import java.util.List;

/**
 * Compares {@link ProtocolWaveletUpdate}s for equality.
 *
 * Generated from org/waveprotocol/box/common/comms/waveclient-rpc.proto. Do not edit.
 */
public final class ProtocolWaveletUpdateUtil {
  private ProtocolWaveletUpdateUtil() {
  }

  /** Returns true if m1 and m2 are structurally equal. */
  public static boolean isEqual(ProtocolWaveletUpdate m1, ProtocolWaveletUpdate m2) {
    if (!m1.getWaveletName().equals(m2.getWaveletName())) return false;
    if (!ProtocolWaveletDeltaUtil.areAllEqual(m1.getAppliedDelta(), m2.getAppliedDelta())) return false;
    if (m1.hasCommitNotice() != m2.hasCommitNotice()) return false;
    if (m1.hasCommitNotice() && !ProtocolHashedVersionUtil.isEqual(m1.getCommitNotice(), m2.getCommitNotice())) return false;
    if (m1.hasResultingVersion() != m2.hasResultingVersion()) return false;
    if (m1.hasResultingVersion() && !ProtocolHashedVersionUtil.isEqual(m1.getResultingVersion(), m2.getResultingVersion())) return false;
    if (m1.hasSnapshot() != m2.hasSnapshot()) return false;
    if (m1.hasSnapshot() && !WaveletSnapshotUtil.isEqual(m1.getSnapshot(), m2.getSnapshot())) return false;
    if (m1.hasMarker() != m2.hasMarker()) return false;
    if (m1.hasMarker() && (m1.getMarker() != m2.getMarker())) return false;
    if (m1.hasChannelId() != m2.hasChannelId()) return false;
    if (m1.hasChannelId() && !m1.getChannelId().equals(m2.getChannelId())) return false;
    return true;
  }

  /** Returns true if m1 and m2 are equal according to isEqual. */
  public static boolean areAllEqual(List<? extends ProtocolWaveletUpdate> m1,
  List<? extends ProtocolWaveletUpdate> m2) {
    if (m1.size() != m2.size()) return false;
    Iterator<? extends ProtocolWaveletUpdate> i1 = m1.iterator();
    Iterator<? extends ProtocolWaveletUpdate> i2 = m2.iterator();
    while (i1.hasNext()) {
      if (!isEqual(i1.next(), i2.next())) return false;
    }
    return true;
  }

  /** Returns a structural hash code of message. */
  public static int getHashCode(ProtocolWaveletUpdate message) {
    int result = 1;
    result = (31 * result) + message.getWaveletName().hashCode();
    result = (31 * result) + message.getAppliedDelta().hashCode();
    result = (31 * result) + (message.hasCommitNotice() ? message.getCommitNotice().hashCode() : 0);
    result = (31 * result) + (message.hasResultingVersion() ? message.getResultingVersion().hashCode() : 0);
    result = (31 * result) + (message.hasSnapshot() ? message.getSnapshot().hashCode() : 0);
    result = (31 * result) + (message.hasMarker() ? Boolean.valueOf(message.getMarker()).hashCode() : 0);
    result = (31 * result) + (message.hasChannelId() ? message.getChannelId().hashCode() : 0);
    return result;
  }

}