/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.common.comms.proto;

import org.waveprotocol.wave.federation.ProtocolWaveletDelta;
import org.waveprotocol.wave.federation.proto.ProtocolWaveletDeltaProtoImpl;
import org.waveprotocol.box.common.comms.ProtocolSubmitRequest;
import org.waveprotocol.box.common.comms.ProtocolSubmitRequestUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of ProtocolSubmitRequest.
 *
 * Generated from org/waveprotocol/box/common/comms/waveclient-rpc.proto. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class ProtocolSubmitRequestProtoImpl
    implements ProtocolSubmitRequest,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.box.common.comms.WaveClientRpc.ProtocolSubmitRequest> {
  private org.waveprotocol.box.common.comms.WaveClientRpc.ProtocolSubmitRequest proto = null;
  private org.waveprotocol.box.common.comms.WaveClientRpc.ProtocolSubmitRequest.Builder protoBuilder = org.waveprotocol.box.common.comms.WaveClientRpc.ProtocolSubmitRequest.newBuilder();
  public ProtocolSubmitRequestProtoImpl() {
  }

  public ProtocolSubmitRequestProtoImpl(org.waveprotocol.box.common.comms.WaveClientRpc.ProtocolSubmitRequest proto) {
    this.proto = proto;
  }

  public ProtocolSubmitRequestProtoImpl(ProtocolSubmitRequest message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.box.common.comms.WaveClientRpc.ProtocolSubmitRequest getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.box.common.comms.WaveClientRpc.ProtocolSubmitRequest proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(ProtocolSubmitRequest message) {
    setWaveletName(message.getWaveletName());
    setDelta(new ProtocolWaveletDeltaProtoImpl(message.getDelta()));
    if (message.hasChannelId()) {
      setChannelId(message.getChannelId());
    } else {
      clearChannelId();
    }
  }

  @Override
  public String getWaveletName() {
    switchToProto();
    return proto.getWaveletName();
  }

  @Override
  public void setWaveletName(String value) {
    switchToProtoBuilder();
    protoBuilder.setWaveletName(value);
  }

  @Override
  public ProtocolWaveletDeltaProtoImpl getDelta() {
    switchToProto();
    return new ProtocolWaveletDeltaProtoImpl(proto.getDelta());
  }

  @Override
  public void setDelta(ProtocolWaveletDelta value) {
    switchToProtoBuilder();
    protoBuilder.clearDelta();
    protoBuilder.setDelta(getOrCreateProtocolWaveletDeltaProtoImpl(value).getPB());
  }

  @Override
  public boolean hasChannelId() {
    switchToProto();
    return proto.hasChannelId();
  }

  @Override
  public void clearChannelId() {
    switchToProtoBuilder();
    protoBuilder.clearChannelId();
  }

  @Override
  public String getChannelId() {
    switchToProto();
    return proto.getChannelId();
  }

  @Override
  public void setChannelId(String value) {
    switchToProtoBuilder();
    protoBuilder.setChannelId(value);
  }

  /** Get or create a ProtocolWaveletDeltaProtoImpl from a ProtocolWaveletDelta. */
  private ProtocolWaveletDeltaProtoImpl getOrCreateProtocolWaveletDeltaProtoImpl(ProtocolWaveletDelta message) {
    if (message instanceof ProtocolWaveletDeltaProtoImpl) {
      return (ProtocolWaveletDeltaProtoImpl) message;
    } else {
      ProtocolWaveletDeltaProtoImpl messageImpl = new ProtocolWaveletDeltaProtoImpl();
      messageImpl.copyFrom(message);
      return messageImpl;
    }
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.box.common.comms.WaveClientRpc.ProtocolSubmitRequest.newBuilder()
          : org.waveprotocol.box.common.comms.WaveClientRpc.ProtocolSubmitRequest.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.box.common.comms.WaveClientRpc.ProtocolSubmitRequest.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    json.add("1", new JsonPrimitive(getWaveletName()));
    {
      JsonElement elem = ((GsonSerializable) getDelta()).toGson(raw, gson);
      json.add("2", elem);
    }
    if (hasChannelId()) {
      json.add("3", new JsonPrimitive(getChannelId()));
    }
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      setWaveletName(elem.getAsString());
    }
    if (jsonObject.has("2")) {
      JsonElement elem = jsonObject.get("2");
      {
        ProtocolWaveletDeltaProtoImpl payload = new ProtocolWaveletDeltaProtoImpl();
        GsonUtil.extractJsonObject(payload, elem, gson, raw);
        setDelta(payload);
      }
    }
    if (jsonObject.has("3")) {
      JsonElement elem = jsonObject.get("3");
      if (!elem.isJsonNull()) {
        setChannelId(elem.getAsString());
      }
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof ProtocolSubmitRequestProtoImpl) {
      return getPB().equals(((ProtocolSubmitRequestProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof ProtocolSubmitRequest) {
      return ProtocolSubmitRequestUtil.isEqual(this, (ProtocolSubmitRequest) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}