/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.profile;

import org.waveprotocol.box.profile.ProfileResponse.FetchedProfile;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for ProfileResponse.
 *
 * Generated from org/waveprotocol/box/profile/profiles.proto. Do not edit.
 */
public interface ProfileResponse {

  public interface FetchedProfile {

    /** Does a deep copy from model. */
    void copyFrom(FetchedProfile model);

    /**
     * Tests if this model is equal to another object.
     * "Equal" is recursively defined as:
     * <ul>
     * <li>both objects implement this interface,</li>
     * <li>all corresponding primitive fields of both objects have the same value, and</li>
     * <li>all corresponding nested-model fields of both objects are "equal".</li>
     * </ul>
     *
     * This is a coarser equivalence than provided by the equals() methods. Two
     * objects may not be equal() to each other, but may be isEqualTo() each other.
     */
    boolean isEqualTo(Object o);

    /** Returns address, or null if hasn't been set. */
    String getAddress();

    /** Sets address. */
    void setAddress(String address);

    /** Returns name, or null if hasn't been set. */
    String getName();

    /** Sets name. */
    void setName(String name);

    /** Returns imageUrl, or null if hasn't been set. */
    String getImageUrl();

    /** Sets imageUrl. */
    void setImageUrl(String imageUrl);

    /** Returns whether profileUrl has been set. */
    boolean hasProfileUrl();

    /** Clears the value of profileUrl. */
    void clearProfileUrl();

    /** Returns profileUrl, or null if hasn't been set. */
    String getProfileUrl();

    /** Sets profileUrl. */
    void setProfileUrl(String profileUrl);
  }

  /** Does a deep copy from model. */
  void copyFrom(ProfileResponse model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns profiles, or null if hasn't been set. */
  List<? extends FetchedProfile> getProfiles();

  /** Adds an element to profiles. */
  void addProfiles(FetchedProfile value);

  /** Adds a list of elements to profiles. */
  void addAllProfiles(List<? extends FetchedProfile> profiles);

  /** Returns the nth element of profiles. */
  FetchedProfile getProfiles(int n);

  /** Sets the nth element of profiles. */
  void setProfiles(int n, FetchedProfile value);

  /** Returns the length of profiles. */
  int getProfilesSize();

  /** Clears profiles. */
  void clearProfiles();
}