/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.profile.proto;

import org.waveprotocol.box.profile.ProfileRequest;
import org.waveprotocol.box.profile.ProfileRequestUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of ProfileRequest.
 *
 * Generated from org/waveprotocol/box/profile/profiles.proto. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class ProfileRequestProtoImpl
    implements ProfileRequest,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.box.profile.ProfilesProto.ProfileRequest> {
  private org.waveprotocol.box.profile.ProfilesProto.ProfileRequest proto = null;
  private org.waveprotocol.box.profile.ProfilesProto.ProfileRequest.Builder protoBuilder = org.waveprotocol.box.profile.ProfilesProto.ProfileRequest.newBuilder();
  public ProfileRequestProtoImpl() {
  }

  public ProfileRequestProtoImpl(org.waveprotocol.box.profile.ProfilesProto.ProfileRequest proto) {
    this.proto = proto;
  }

  public ProfileRequestProtoImpl(ProfileRequest message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.box.profile.ProfilesProto.ProfileRequest getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.box.profile.ProfilesProto.ProfileRequest proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(ProfileRequest message) {
    clearAddresses();
    for (String field : message.getAddresses()) {
      addAddresses(field);
    }
  }

  @Override
  public List<String> getAddresses() {
    switchToProto();
    return Collections.unmodifiableList(proto.getAddressesList());
  }

  @Override
  public void addAllAddresses(List<String> values) {
    switchToProtoBuilder();
    protoBuilder.addAllAddresses(values);
  }

  @Override
  public String getAddresses(int n) {
    switchToProto();
    return proto.getAddresses(n);
  }

  @Override
  public void setAddresses(int n, String value) {
    switchToProtoBuilder();
    protoBuilder.setAddresses(n, value);
  }

  @Override
  public int getAddressesSize() {
    switchToProto();
    return proto.getAddressesCount();
  }

  @Override
  public void addAddresses(String value) {
    switchToProtoBuilder();
    protoBuilder.addAddresses(value);
  }

  @Override
  public void clearAddresses() {
    switchToProtoBuilder();
    protoBuilder.clearAddresses();
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.box.profile.ProfilesProto.ProfileRequest.newBuilder()
          : org.waveprotocol.box.profile.ProfilesProto.ProfileRequest.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.box.profile.ProfilesProto.ProfileRequest.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    {
      JsonArray array = new JsonArray();
      for (int i = 0; i < getAddressesSize(); i++) {
        array.add(new JsonPrimitive(getAddresses(i)));
      }
      json.add("1", array);
    }
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      {
        JsonArray array = elem.getAsJsonArray();
        for (int i = 0; i < array.size(); i++) {
          addAddresses(array.get(i).getAsString());
        }
      }
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof ProfileRequestProtoImpl) {
      return getPB().equals(((ProfileRequestProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof ProfileRequest) {
      return ProfileRequestUtil.isEqual(this, (ProfileRequest) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}