/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.search;

import org.waveprotocol.box.search.SearchResponse.Digest;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for SearchResponse.
 *
 * Generated from org/waveprotocol/box/search/search.proto. Do not edit.
 */
public interface SearchResponse {

  public interface Digest {

    /** Does a deep copy from model. */
    void copyFrom(Digest model);

    /**
     * Tests if this model is equal to another object.
     * "Equal" is recursively defined as:
     * <ul>
     * <li>both objects implement this interface,</li>
     * <li>all corresponding primitive fields of both objects have the same value, and</li>
     * <li>all corresponding nested-model fields of both objects are "equal".</li>
     * </ul>
     *
     * This is a coarser equivalence than provided by the equals() methods. Two
     * objects may not be equal() to each other, but may be isEqualTo() each other.
     */
    boolean isEqualTo(Object o);

    /** Returns title, or null if hasn't been set. */
    String getTitle();

    /** Sets title. */
    void setTitle(String title);

    /** Returns snippet, or null if hasn't been set. */
    String getSnippet();

    /** Sets snippet. */
    void setSnippet(String snippet);

    /** Returns waveId, or null if hasn't been set. */
    String getWaveId();

    /** Sets waveId. */
    void setWaveId(String waveId);

    /** Returns lastModified, or null if hasn't been set. */
    long getLastModified();

    /** Sets lastModified. */
    void setLastModified(long lastModified);

    /** Returns unreadCount, or null if hasn't been set. */
    int getUnreadCount();

    /** Sets unreadCount. */
    void setUnreadCount(int unreadCount);

    /** Returns blipCount, or null if hasn't been set. */
    int getBlipCount();

    /** Sets blipCount. */
    void setBlipCount(int blipCount);

    /** Returns participants, or null if hasn't been set. */
    List<String> getParticipants();

    /** Adds an element to participants. */
    void addParticipants(String value);

    /** Adds a list of elements to participants. */
    void addAllParticipants(List<String> participants);

    /** Returns the nth element of participants. */
    String getParticipants(int n);

    /** Sets the nth element of participants. */
    void setParticipants(int n, String value);

    /** Returns the length of participants. */
    int getParticipantsSize();

    /** Clears participants. */
    void clearParticipants();

    /** Returns author, or null if hasn't been set. */
    String getAuthor();

    /** Sets author. */
    void setAuthor(String author);
  }

  /** Does a deep copy from model. */
  void copyFrom(SearchResponse model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns query, or null if hasn't been set. */
  String getQuery();

  /** Sets query. */
  void setQuery(String query);

  /** Returns totalResults, or null if hasn't been set. */
  int getTotalResults();

  /** Sets totalResults. */
  void setTotalResults(int totalResults);

  /** Returns digests, or null if hasn't been set. */
  List<? extends Digest> getDigests();

  /** Adds an element to digests. */
  void addDigests(Digest value);

  /** Adds a list of elements to digests. */
  void addAllDigests(List<? extends Digest> digests);

  /** Returns the nth element of digests. */
  Digest getDigests(int n);

  /** Sets the nth element of digests. */
  void setDigests(int n, Digest value);

  /** Returns the length of digests. */
  int getDigestsSize();

  /** Clears digests. */
  void clearDigests();
}