/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.search.proto;

import org.waveprotocol.box.search.SearchRequest;
import org.waveprotocol.box.search.SearchRequestUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of SearchRequest.
 *
 * Generated from org/waveprotocol/box/search/search.proto. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class SearchRequestProtoImpl
    implements SearchRequest,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.box.search.SearchProto.SearchRequest> {
  private org.waveprotocol.box.search.SearchProto.SearchRequest proto = null;
  private org.waveprotocol.box.search.SearchProto.SearchRequest.Builder protoBuilder = org.waveprotocol.box.search.SearchProto.SearchRequest.newBuilder();
  public SearchRequestProtoImpl() {
  }

  public SearchRequestProtoImpl(org.waveprotocol.box.search.SearchProto.SearchRequest proto) {
    this.proto = proto;
  }

  public SearchRequestProtoImpl(SearchRequest message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.box.search.SearchProto.SearchRequest getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.box.search.SearchProto.SearchRequest proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(SearchRequest message) {
    setQuery(message.getQuery());
    setIndex(message.getIndex());
    setNumResults(message.getNumResults());
  }

  @Override
  public String getQuery() {
    switchToProto();
    return proto.getQuery();
  }

  @Override
  public void setQuery(String value) {
    switchToProtoBuilder();
    protoBuilder.setQuery(value);
  }

  @Override
  public int getIndex() {
    switchToProto();
    return proto.getIndex();
  }

  @Override
  public void setIndex(int value) {
    switchToProtoBuilder();
    protoBuilder.setIndex(value);
  }

  @Override
  public int getNumResults() {
    switchToProto();
    return proto.getNumResults();
  }

  @Override
  public void setNumResults(int value) {
    switchToProtoBuilder();
    protoBuilder.setNumResults(value);
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.box.search.SearchProto.SearchRequest.newBuilder()
          : org.waveprotocol.box.search.SearchProto.SearchRequest.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.box.search.SearchProto.SearchRequest.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    json.add("1", new JsonPrimitive(getQuery()));
    json.add("2", new JsonPrimitive(getIndex()));
    json.add("3", new JsonPrimitive(getNumResults()));
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      setQuery(elem.getAsString());
    }
    if (jsonObject.has("2")) {
      JsonElement elem = jsonObject.get("2");
      setIndex(elem.getAsInt());
    }
    if (jsonObject.has("3")) {
      JsonElement elem = jsonObject.get("3");
      setNumResults(elem.getAsInt());
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof SearchRequestProtoImpl) {
      return getPB().equals(((SearchRequestProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof SearchRequest) {
      return SearchRequestUtil.isEqual(this, (SearchRequest) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}