/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.box.server.rpc.proto;

import org.waveprotocol.box.server.rpc.RpcFinished;
import org.waveprotocol.box.server.rpc.RpcFinishedUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of RpcFinished.
 *
 * Generated from org/waveprotocol/box/server/rpc/rpc.proto. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class RpcFinishedProtoImpl
    implements RpcFinished,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.box.server.rpc.Rpc.RpcFinished> {
  private org.waveprotocol.box.server.rpc.Rpc.RpcFinished proto = null;
  private org.waveprotocol.box.server.rpc.Rpc.RpcFinished.Builder protoBuilder = org.waveprotocol.box.server.rpc.Rpc.RpcFinished.newBuilder();
  public RpcFinishedProtoImpl() {
  }

  public RpcFinishedProtoImpl(org.waveprotocol.box.server.rpc.Rpc.RpcFinished proto) {
    this.proto = proto;
  }

  public RpcFinishedProtoImpl(RpcFinished message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.box.server.rpc.Rpc.RpcFinished getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.box.server.rpc.Rpc.RpcFinished proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(RpcFinished message) {
    setFailed(message.getFailed());
    if (message.hasErrorText()) {
      setErrorText(message.getErrorText());
    } else {
      clearErrorText();
    }
  }

  @Override
  public boolean getFailed() {
    switchToProto();
    return proto.getFailed();
  }

  @Override
  public void setFailed(boolean value) {
    switchToProtoBuilder();
    protoBuilder.setFailed(value);
  }

  @Override
  public boolean hasErrorText() {
    switchToProto();
    return proto.hasErrorText();
  }

  @Override
  public void clearErrorText() {
    switchToProtoBuilder();
    protoBuilder.clearErrorText();
  }

  @Override
  public String getErrorText() {
    switchToProto();
    return proto.getErrorText();
  }

  @Override
  public void setErrorText(String value) {
    switchToProtoBuilder();
    protoBuilder.setErrorText(value);
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.box.server.rpc.Rpc.RpcFinished.newBuilder()
          : org.waveprotocol.box.server.rpc.Rpc.RpcFinished.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.box.server.rpc.Rpc.RpcFinished.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    json.add("1", new JsonPrimitive(getFailed()));
    if (hasErrorText()) {
      json.add("2", new JsonPrimitive(getErrorText()));
    }
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      setFailed(elem.getAsBoolean());
    }
    if (jsonObject.has("2")) {
      JsonElement elem = jsonObject.get("2");
      if (!elem.isJsonNull()) {
        setErrorText(elem.getAsString());
      }
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof RpcFinishedProtoImpl) {
      return getPB().equals(((RpcFinishedProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof RpcFinished) {
      return RpcFinishedUtil.isEqual(this, (RpcFinished) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}