/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.concurrencycontrol;

import org.waveprotocol.wave.concurrencycontrol.ResponseStatus;
import org.waveprotocol.wave.concurrencycontrol.FetchWaveViewResponse.Wavelet;
import org.waveprotocol.wave.concurrencycontrol.WaveletSnapshot;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for FetchWaveViewResponse.
 *
 * Generated from org/waveprotocol/wave/concurrencycontrol/clientserver.proto. Do not edit.
 */
public interface FetchWaveViewResponse {

  public interface Wavelet {

    /** Does a deep copy from model. */
    void copyFrom(Wavelet model);

    /**
     * Tests if this model is equal to another object.
     * "Equal" is recursively defined as:
     * <ul>
     * <li>both objects implement this interface,</li>
     * <li>all corresponding primitive fields of both objects have the same value, and</li>
     * <li>all corresponding nested-model fields of both objects are "equal".</li>
     * </ul>
     *
     * This is a coarser equivalence than provided by the equals() methods. Two
     * objects may not be equal() to each other, but may be isEqualTo() each other.
     */
    boolean isEqualTo(Object o);

    /** Returns waveletId, or null if hasn't been set. */
    String getWaveletId();

    /** Sets waveletId. */
    void setWaveletId(String waveletId);

    /** Returns whether snapshot has been set. */
    boolean hasSnapshot();

    /** Clears the value of snapshot. */
    void clearSnapshot();

    /** Returns snapshot, or null if hasn't been set. */
    WaveletSnapshot getSnapshot();

    /** Sets snapshot. */
    void setSnapshot(WaveletSnapshot snapshot);
  }

  /** Does a deep copy from model. */
  void copyFrom(FetchWaveViewResponse model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns status, or null if hasn't been set. */
  ResponseStatus getStatus();

  /** Sets status. */
  void setStatus(ResponseStatus status);

  /** Returns wavelet, or null if hasn't been set. */
  List<? extends Wavelet> getWavelet();

  /** Adds an element to wavelet. */
  void addWavelet(Wavelet value);

  /** Adds a list of elements to wavelet. */
  void addAllWavelet(List<? extends Wavelet> wavelet);

  /** Returns the nth element of wavelet. */
  Wavelet getWavelet(int n);

  /** Sets the nth element of wavelet. */
  void setWavelet(int n, Wavelet value);

  /** Returns the length of wavelet. */
  int getWaveletSize();

  /** Clears wavelet. */
  void clearWavelet();
}