/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.concurrencycontrol;

import org.waveprotocol.wave.concurrencycontrol.OpenWaveletChannelStream.*;
import org.waveprotocol.wave.concurrencycontrol.WaveletUpdate;
import org.waveprotocol.wave.federation.ProtocolHashedVersion;
import org.waveprotocol.wave.concurrencycontrol.WaveletChannelTerminator;
import org.waveprotocol.wave.concurrencycontrol.WaveletUpdateUtil;
import org.waveprotocol.wave.federation.ProtocolHashedVersionUtil;
import org.waveprotocol.wave.concurrencycontrol.WaveletChannelTerminatorUtil;
import org.waveprotocol.wave.communication.Blob;
import java.util.Iterator;
import java.util.List;

/**
 * Compares {@link OpenWaveletChannelStream}s for equality.
 *
 * Generated from org/waveprotocol/wave/concurrencycontrol/clientserver.proto. Do not edit.
 */
public final class OpenWaveletChannelStreamUtil {
  private OpenWaveletChannelStreamUtil() {
  }

  /** Returns true if m1 and m2 are structurally equal. */
  public static boolean isEqual(OpenWaveletChannelStream m1, OpenWaveletChannelStream m2) {
    if (m1.hasChannelId() != m2.hasChannelId()) return false;
    if (m1.hasChannelId() && !m1.getChannelId().equals(m2.getChannelId())) return false;
    if (m1.hasDelta() != m2.hasDelta()) return false;
    if (m1.hasDelta() && !WaveletUpdateUtil.isEqual(m1.getDelta(), m2.getDelta())) return false;
    if (m1.hasCommitVersion() != m2.hasCommitVersion()) return false;
    if (m1.hasCommitVersion() && !ProtocolHashedVersionUtil.isEqual(m1.getCommitVersion(), m2.getCommitVersion())) return false;
    if (m1.hasTerminator() != m2.hasTerminator()) return false;
    if (m1.hasTerminator() && !WaveletChannelTerminatorUtil.isEqual(m1.getTerminator(), m2.getTerminator())) return false;
    return true;
  }

  /** Returns true if m1 and m2 are equal according to isEqual. */
  public static boolean areAllEqual(List<? extends OpenWaveletChannelStream> m1,
  List<? extends OpenWaveletChannelStream> m2) {
    if (m1.size() != m2.size()) return false;
    Iterator<? extends OpenWaveletChannelStream> i1 = m1.iterator();
    Iterator<? extends OpenWaveletChannelStream> i2 = m2.iterator();
    while (i1.hasNext()) {
      if (!isEqual(i1.next(), i2.next())) return false;
    }
    return true;
  }

  /** Returns a structural hash code of message. */
  public static int getHashCode(OpenWaveletChannelStream message) {
    int result = 1;
    result = (31 * result) + (message.hasChannelId() ? message.getChannelId().hashCode() : 0);
    result = (31 * result) + (message.hasDelta() ? message.getDelta().hashCode() : 0);
    result = (31 * result) + (message.hasCommitVersion() ? message.getCommitVersion().hashCode() : 0);
    result = (31 * result) + (message.hasTerminator() ? message.getTerminator().hashCode() : 0);
    return result;
  }

}