/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.concurrencycontrol;

import org.waveprotocol.wave.concurrencycontrol.ResponseStatus.ResponseCode;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for ResponseStatus.
 *
 * Generated from org/waveprotocol/wave/concurrencycontrol/clientserver.proto. Do not edit.
 */
public interface ResponseStatus {

  enum ResponseCode implements ProtoEnums.HasIntValue {
    OK(0),
    BAD_REQUEST(1),
    INTERNAL_ERROR(2),
    NOT_AUTHORIZED(3),
    VERSION_ERROR(4),
    INVALID_OPERATION(5),
    SCHEMA_VIOLATION(6),
    SIZE_LIMIT_EXCEEDED(7),
    POLICY_VIOLATION(8),
    QUARANTINED(9),
    TOO_OLD(10),
    UNKNOWN(-1);
    private final int value;
    private ResponseCode(int value) {
      this.value = value;
    }

    @Override
    public int getValue() {
      return value;
    }

  }

  /** Does a deep copy from model. */
  void copyFrom(ResponseStatus model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns status, or null if hasn't been set. */
  ResponseCode getStatus();

  /** Sets status. */
  void setStatus(ResponseCode status);

  /** Returns whether failureReason has been set. */
  boolean hasFailureReason();

  /** Clears the value of failureReason. */
  void clearFailureReason();

  /** Returns failureReason, or null if hasn't been set. */
  String getFailureReason();

  /** Sets failureReason. */
  void setFailureReason(String failureReason);
}