/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.concurrencycontrol;

import org.waveprotocol.wave.concurrencycontrol.DocumentSnapshot;
import org.waveprotocol.wave.federation.ProtocolHashedVersion;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for WaveletSnapshot.
 *
 * Generated from org/waveprotocol/wave/concurrencycontrol/clientserver.proto. Do not edit.
 */
public interface WaveletSnapshot {

  /** Does a deep copy from model. */
  void copyFrom(WaveletSnapshot model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns waveletId, or null if hasn't been set. */
  String getWaveletId();

  /** Sets waveletId. */
  void setWaveletId(String waveletId);

  /** Returns participant, or null if hasn't been set. */
  List<String> getParticipant();

  /** Adds an element to participant. */
  void addParticipant(String value);

  /** Adds a list of elements to participant. */
  void addAllParticipant(List<String> participant);

  /** Returns the nth element of participant. */
  String getParticipant(int n);

  /** Sets the nth element of participant. */
  void setParticipant(int n, String value);

  /** Returns the length of participant. */
  int getParticipantSize();

  /** Clears participant. */
  void clearParticipant();

  /** Returns document, or null if hasn't been set. */
  List<? extends DocumentSnapshot> getDocument();

  /** Adds an element to document. */
  void addDocument(DocumentSnapshot value);

  /** Adds a list of elements to document. */
  void addAllDocument(List<? extends DocumentSnapshot> document);

  /** Returns the nth element of document. */
  DocumentSnapshot getDocument(int n);

  /** Sets the nth element of document. */
  void setDocument(int n, DocumentSnapshot value);

  /** Returns the length of document. */
  int getDocumentSize();

  /** Clears document. */
  void clearDocument();

  /** Returns version, or null if hasn't been set. */
  ProtocolHashedVersion getVersion();

  /** Sets version. */
  void setVersion(ProtocolHashedVersion version);

  /** Returns lastModifiedTime, or null if hasn't been set. */
  long getLastModifiedTime();

  /** Sets lastModifiedTime. */
  void setLastModifiedTime(long lastModifiedTime);

  /** Returns creator, or null if hasn't been set. */
  String getCreator();

  /** Sets creator. */
  void setCreator(String creator);

  /** Returns creationTime, or null if hasn't been set. */
  long getCreationTime();

  /** Sets creationTime. */
  void setCreationTime(long creationTime);
}