/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.concurrencycontrol.proto;

import org.waveprotocol.wave.federation.ProtocolHashedVersion;
import org.waveprotocol.wave.federation.proto.ProtocolHashedVersionProtoImpl;
import org.waveprotocol.wave.concurrencycontrol.OpenWaveletChannelRequest;
import org.waveprotocol.wave.concurrencycontrol.OpenWaveletChannelRequestUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of OpenWaveletChannelRequest.
 *
 * Generated from org/waveprotocol/wave/concurrencycontrol/clientserver.proto. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class OpenWaveletChannelRequestProtoImpl
    implements OpenWaveletChannelRequest,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelRequest> {
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelRequest proto = null;
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelRequest.Builder protoBuilder = org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelRequest.newBuilder();
  public OpenWaveletChannelRequestProtoImpl() {
  }

  public OpenWaveletChannelRequestProtoImpl(org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelRequest proto) {
    this.proto = proto;
  }

  public OpenWaveletChannelRequestProtoImpl(OpenWaveletChannelRequest message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelRequest getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelRequest proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(OpenWaveletChannelRequest message) {
    setWaveId(message.getWaveId());
    setWaveletId(message.getWaveletId());
    setBeginVersion(new ProtocolHashedVersionProtoImpl(message.getBeginVersion()));
  }

  @Override
  public String getWaveId() {
    switchToProto();
    return proto.getWaveId();
  }

  @Override
  public void setWaveId(String value) {
    switchToProtoBuilder();
    protoBuilder.setWaveId(value);
  }

  @Override
  public String getWaveletId() {
    switchToProto();
    return proto.getWaveletId();
  }

  @Override
  public void setWaveletId(String value) {
    switchToProtoBuilder();
    protoBuilder.setWaveletId(value);
  }

  @Override
  public ProtocolHashedVersionProtoImpl getBeginVersion() {
    switchToProto();
    return new ProtocolHashedVersionProtoImpl(proto.getBeginVersion());
  }

  @Override
  public void setBeginVersion(ProtocolHashedVersion value) {
    switchToProtoBuilder();
    protoBuilder.clearBeginVersion();
    protoBuilder.setBeginVersion(getOrCreateProtocolHashedVersionProtoImpl(value).getPB());
  }

  /** Get or create a ProtocolHashedVersionProtoImpl from a ProtocolHashedVersion. */
  private ProtocolHashedVersionProtoImpl getOrCreateProtocolHashedVersionProtoImpl(ProtocolHashedVersion message) {
    if (message instanceof ProtocolHashedVersionProtoImpl) {
      return (ProtocolHashedVersionProtoImpl) message;
    } else {
      ProtocolHashedVersionProtoImpl messageImpl = new ProtocolHashedVersionProtoImpl();
      messageImpl.copyFrom(message);
      return messageImpl;
    }
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelRequest.newBuilder()
          : org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelRequest.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelRequest.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    json.add("1", new JsonPrimitive(getWaveId()));
    json.add("2", new JsonPrimitive(getWaveletId()));
    {
      JsonElement elem = ((GsonSerializable) getBeginVersion()).toGson(raw, gson);
      json.add("3", elem);
    }
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      setWaveId(elem.getAsString());
    }
    if (jsonObject.has("2")) {
      JsonElement elem = jsonObject.get("2");
      setWaveletId(elem.getAsString());
    }
    if (jsonObject.has("3")) {
      JsonElement elem = jsonObject.get("3");
      {
        ProtocolHashedVersionProtoImpl payload = new ProtocolHashedVersionProtoImpl();
        GsonUtil.extractJsonObject(payload, elem, gson, raw);
        setBeginVersion(payload);
      }
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof OpenWaveletChannelRequestProtoImpl) {
      return getPB().equals(((OpenWaveletChannelRequestProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof OpenWaveletChannelRequest) {
      return OpenWaveletChannelRequestUtil.isEqual(this, (OpenWaveletChannelRequest) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}