/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.concurrencycontrol.proto;

import org.waveprotocol.wave.concurrencycontrol.WaveletUpdate;
import org.waveprotocol.wave.federation.ProtocolHashedVersion;
import org.waveprotocol.wave.concurrencycontrol.WaveletChannelTerminator;
import org.waveprotocol.wave.concurrencycontrol.proto.WaveletUpdateProtoImpl;
import org.waveprotocol.wave.federation.proto.ProtocolHashedVersionProtoImpl;
import org.waveprotocol.wave.concurrencycontrol.proto.WaveletChannelTerminatorProtoImpl;
import org.waveprotocol.wave.concurrencycontrol.OpenWaveletChannelStream;
import org.waveprotocol.wave.concurrencycontrol.OpenWaveletChannelStreamUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of OpenWaveletChannelStream.
 *
 * Generated from org/waveprotocol/wave/concurrencycontrol/clientserver.proto. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class OpenWaveletChannelStreamProtoImpl
    implements OpenWaveletChannelStream,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelStream> {
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelStream proto = null;
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelStream.Builder protoBuilder = org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelStream.newBuilder();
  public OpenWaveletChannelStreamProtoImpl() {
  }

  public OpenWaveletChannelStreamProtoImpl(org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelStream proto) {
    this.proto = proto;
  }

  public OpenWaveletChannelStreamProtoImpl(OpenWaveletChannelStream message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelStream getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelStream proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(OpenWaveletChannelStream message) {
    if (message.hasChannelId()) {
      setChannelId(message.getChannelId());
    } else {
      clearChannelId();
    }
    if (message.hasDelta()) {
      setDelta(new WaveletUpdateProtoImpl(message.getDelta()));
    } else {
      clearDelta();
    }
    if (message.hasCommitVersion()) {
      setCommitVersion(new ProtocolHashedVersionProtoImpl(message.getCommitVersion()));
    } else {
      clearCommitVersion();
    }
    if (message.hasTerminator()) {
      setTerminator(new WaveletChannelTerminatorProtoImpl(message.getTerminator()));
    } else {
      clearTerminator();
    }
  }

  @Override
  public boolean hasChannelId() {
    switchToProto();
    return proto.hasChannelId();
  }

  @Override
  public void clearChannelId() {
    switchToProtoBuilder();
    protoBuilder.clearChannelId();
  }

  @Override
  public String getChannelId() {
    switchToProto();
    return proto.getChannelId();
  }

  @Override
  public void setChannelId(String value) {
    switchToProtoBuilder();
    protoBuilder.setChannelId(value);
  }

  @Override
  public boolean hasDelta() {
    switchToProto();
    return proto.hasDelta();
  }

  @Override
  public void clearDelta() {
    switchToProtoBuilder();
    protoBuilder.clearDelta();
  }

  @Override
  public WaveletUpdateProtoImpl getDelta() {
    switchToProto();
    return new WaveletUpdateProtoImpl(proto.getDelta());
  }

  @Override
  public void setDelta(WaveletUpdate value) {
    switchToProtoBuilder();
    protoBuilder.clearDelta();
    protoBuilder.setDelta(getOrCreateWaveletUpdateProtoImpl(value).getPB());
  }

  @Override
  public boolean hasCommitVersion() {
    switchToProto();
    return proto.hasCommitVersion();
  }

  @Override
  public void clearCommitVersion() {
    switchToProtoBuilder();
    protoBuilder.clearCommitVersion();
  }

  @Override
  public ProtocolHashedVersionProtoImpl getCommitVersion() {
    switchToProto();
    return new ProtocolHashedVersionProtoImpl(proto.getCommitVersion());
  }

  @Override
  public void setCommitVersion(ProtocolHashedVersion value) {
    switchToProtoBuilder();
    protoBuilder.clearCommitVersion();
    protoBuilder.setCommitVersion(getOrCreateProtocolHashedVersionProtoImpl(value).getPB());
  }

  @Override
  public boolean hasTerminator() {
    switchToProto();
    return proto.hasTerminator();
  }

  @Override
  public void clearTerminator() {
    switchToProtoBuilder();
    protoBuilder.clearTerminator();
  }

  @Override
  public WaveletChannelTerminatorProtoImpl getTerminator() {
    switchToProto();
    return new WaveletChannelTerminatorProtoImpl(proto.getTerminator());
  }

  @Override
  public void setTerminator(WaveletChannelTerminator value) {
    switchToProtoBuilder();
    protoBuilder.clearTerminator();
    protoBuilder.setTerminator(getOrCreateWaveletChannelTerminatorProtoImpl(value).getPB());
  }

  /** Get or create a WaveletUpdateProtoImpl from a WaveletUpdate. */
  private WaveletUpdateProtoImpl getOrCreateWaveletUpdateProtoImpl(WaveletUpdate message) {
    if (message instanceof WaveletUpdateProtoImpl) {
      return (WaveletUpdateProtoImpl) message;
    } else {
      WaveletUpdateProtoImpl messageImpl = new WaveletUpdateProtoImpl();
      messageImpl.copyFrom(message);
      return messageImpl;
    }
  }

  /** Get or create a ProtocolHashedVersionProtoImpl from a ProtocolHashedVersion. */
  private ProtocolHashedVersionProtoImpl getOrCreateProtocolHashedVersionProtoImpl(ProtocolHashedVersion message) {
    if (message instanceof ProtocolHashedVersionProtoImpl) {
      return (ProtocolHashedVersionProtoImpl) message;
    } else {
      ProtocolHashedVersionProtoImpl messageImpl = new ProtocolHashedVersionProtoImpl();
      messageImpl.copyFrom(message);
      return messageImpl;
    }
  }

  /** Get or create a WaveletChannelTerminatorProtoImpl from a WaveletChannelTerminator. */
  private WaveletChannelTerminatorProtoImpl getOrCreateWaveletChannelTerminatorProtoImpl(WaveletChannelTerminator message) {
    if (message instanceof WaveletChannelTerminatorProtoImpl) {
      return (WaveletChannelTerminatorProtoImpl) message;
    } else {
      WaveletChannelTerminatorProtoImpl messageImpl = new WaveletChannelTerminatorProtoImpl();
      messageImpl.copyFrom(message);
      return messageImpl;
    }
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelStream.newBuilder()
          : org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelStream.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.wave.concurrencycontrol.ClientServer.OpenWaveletChannelStream.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    if (hasChannelId()) {
      json.add("1", new JsonPrimitive(getChannelId()));
    }
    if (hasDelta()) {
      {
        JsonElement elem = ((GsonSerializable) getDelta()).toGson(raw, gson);
        json.add("2", elem);
      }
    }
    if (hasCommitVersion()) {
      {
        JsonElement elem = ((GsonSerializable) getCommitVersion()).toGson(raw, gson);
        json.add("3", elem);
      }
    }
    if (hasTerminator()) {
      {
        JsonElement elem = ((GsonSerializable) getTerminator()).toGson(raw, gson);
        json.add("4", elem);
      }
    }
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      if (!elem.isJsonNull()) {
        setChannelId(elem.getAsString());
      }
    }
    if (jsonObject.has("2")) {
      JsonElement elem = jsonObject.get("2");
      if (!elem.isJsonNull()) {
        {
          WaveletUpdateProtoImpl payload = new WaveletUpdateProtoImpl();
          GsonUtil.extractJsonObject(payload, elem, gson, raw);
          setDelta(payload);
        }
      }
    }
    if (jsonObject.has("3")) {
      JsonElement elem = jsonObject.get("3");
      if (!elem.isJsonNull()) {
        {
          ProtocolHashedVersionProtoImpl payload = new ProtocolHashedVersionProtoImpl();
          GsonUtil.extractJsonObject(payload, elem, gson, raw);
          setCommitVersion(payload);
        }
      }
    }
    if (jsonObject.has("4")) {
      JsonElement elem = jsonObject.get("4");
      if (!elem.isJsonNull()) {
        {
          WaveletChannelTerminatorProtoImpl payload = new WaveletChannelTerminatorProtoImpl();
          GsonUtil.extractJsonObject(payload, elem, gson, raw);
          setTerminator(payload);
        }
      }
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof OpenWaveletChannelStreamProtoImpl) {
      return getPB().equals(((OpenWaveletChannelStreamProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof OpenWaveletChannelStream) {
      return OpenWaveletChannelStreamUtil.isEqual(this, (OpenWaveletChannelStream) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}