/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.concurrencycontrol.proto;

import org.waveprotocol.wave.concurrencycontrol.ResponseStatus.ResponseCode;
import org.waveprotocol.wave.concurrencycontrol.ResponseStatus;
import org.waveprotocol.wave.concurrencycontrol.ResponseStatusUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of ResponseStatus.
 *
 * Generated from org/waveprotocol/wave/concurrencycontrol/clientserver.proto. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class ResponseStatusProtoImpl
    implements ResponseStatus,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus> {
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus proto = null;
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus.Builder protoBuilder = org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus.newBuilder();
  public ResponseStatusProtoImpl() {
  }

  public ResponseStatusProtoImpl(org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus proto) {
    this.proto = proto;
  }

  public ResponseStatusProtoImpl(ResponseStatus message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(ResponseStatus message) {
    setStatus(message.getStatus());
    if (message.hasFailureReason()) {
      setFailureReason(message.getFailureReason());
    } else {
      clearFailureReason();
    }
  }

  @Override
  public ResponseCode getStatus() {
    switchToProto();
    return toPojoEnumResponseCode(proto.getStatus());
  }

  @Override
  public void setStatus(ResponseCode value) {
    switchToProtoBuilder();
    protoBuilder.setStatus(toProtoEnumResponseCode(value));
  }

  @Override
  public boolean hasFailureReason() {
    switchToProto();
    return proto.hasFailureReason();
  }

  @Override
  public void clearFailureReason() {
    switchToProtoBuilder();
    protoBuilder.clearFailureReason();
  }

  @Override
  public String getFailureReason() {
    switchToProto();
    return proto.getFailureReason();
  }

  @Override
  public void setFailureReason(String value) {
    switchToProtoBuilder();
    protoBuilder.setFailureReason(value);
  }

  /** Translates a pojo enum to a proto enum. */
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus.ResponseCode toProtoEnumResponseCode(ResponseCode value) {
    return org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus.ResponseCode.valueOf(value.getValue());
  }

  /** Translates a proto enum to a pojo enum. */
  private ResponseCode toPojoEnumResponseCode(org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus.ResponseCode value) {
    return ProtoEnums.valOf(value.getNumber(), ResponseCode.values());
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus.newBuilder()
          : org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.wave.concurrencycontrol.ClientServer.ResponseStatus.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    json.add("1", new JsonPrimitive(getStatus().getValue()));
    if (hasFailureReason()) {
      json.add("2", new JsonPrimitive(getFailureReason()));
    }
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      setStatus(ProtoEnums.valOf(elem.getAsInt(), ResponseCode.values()));
    }
    if (jsonObject.has("2")) {
      JsonElement elem = jsonObject.get("2");
      if (!elem.isJsonNull()) {
        setFailureReason(elem.getAsString());
      }
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof ResponseStatusProtoImpl) {
      return getPB().equals(((ResponseStatusProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof ResponseStatus) {
      return ResponseStatusUtil.isEqual(this, (ResponseStatus) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}