/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.concurrencycontrol.proto;

import org.waveprotocol.wave.federation.ProtocolWaveletDelta;
import org.waveprotocol.wave.federation.proto.ProtocolWaveletDeltaProtoImpl;
import org.waveprotocol.wave.concurrencycontrol.SubmitDeltaRequest;
import org.waveprotocol.wave.concurrencycontrol.SubmitDeltaRequestUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of SubmitDeltaRequest.
 *
 * Generated from org/waveprotocol/wave/concurrencycontrol/clientserver.proto. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class SubmitDeltaRequestProtoImpl
    implements SubmitDeltaRequest,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.wave.concurrencycontrol.ClientServer.SubmitDeltaRequest> {
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.SubmitDeltaRequest proto = null;
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.SubmitDeltaRequest.Builder protoBuilder = org.waveprotocol.wave.concurrencycontrol.ClientServer.SubmitDeltaRequest.newBuilder();
  public SubmitDeltaRequestProtoImpl() {
  }

  public SubmitDeltaRequestProtoImpl(org.waveprotocol.wave.concurrencycontrol.ClientServer.SubmitDeltaRequest proto) {
    this.proto = proto;
  }

  public SubmitDeltaRequestProtoImpl(SubmitDeltaRequest message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.wave.concurrencycontrol.ClientServer.SubmitDeltaRequest getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.wave.concurrencycontrol.ClientServer.SubmitDeltaRequest proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(SubmitDeltaRequest message) {
    setWaveId(message.getWaveId());
    setWaveletId(message.getWaveletId());
    setDelta(new ProtocolWaveletDeltaProtoImpl(message.getDelta()));
    setChannelId(message.getChannelId());
  }

  @Override
  public String getWaveId() {
    switchToProto();
    return proto.getWaveId();
  }

  @Override
  public void setWaveId(String value) {
    switchToProtoBuilder();
    protoBuilder.setWaveId(value);
  }

  @Override
  public String getWaveletId() {
    switchToProto();
    return proto.getWaveletId();
  }

  @Override
  public void setWaveletId(String value) {
    switchToProtoBuilder();
    protoBuilder.setWaveletId(value);
  }

  @Override
  public ProtocolWaveletDeltaProtoImpl getDelta() {
    switchToProto();
    return new ProtocolWaveletDeltaProtoImpl(proto.getDelta());
  }

  @Override
  public void setDelta(ProtocolWaveletDelta value) {
    switchToProtoBuilder();
    protoBuilder.clearDelta();
    protoBuilder.setDelta(getOrCreateProtocolWaveletDeltaProtoImpl(value).getPB());
  }

  @Override
  public String getChannelId() {
    switchToProto();
    return proto.getChannelId();
  }

  @Override
  public void setChannelId(String value) {
    switchToProtoBuilder();
    protoBuilder.setChannelId(value);
  }

  /** Get or create a ProtocolWaveletDeltaProtoImpl from a ProtocolWaveletDelta. */
  private ProtocolWaveletDeltaProtoImpl getOrCreateProtocolWaveletDeltaProtoImpl(ProtocolWaveletDelta message) {
    if (message instanceof ProtocolWaveletDeltaProtoImpl) {
      return (ProtocolWaveletDeltaProtoImpl) message;
    } else {
      ProtocolWaveletDeltaProtoImpl messageImpl = new ProtocolWaveletDeltaProtoImpl();
      messageImpl.copyFrom(message);
      return messageImpl;
    }
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.wave.concurrencycontrol.ClientServer.SubmitDeltaRequest.newBuilder()
          : org.waveprotocol.wave.concurrencycontrol.ClientServer.SubmitDeltaRequest.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.wave.concurrencycontrol.ClientServer.SubmitDeltaRequest.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    json.add("1", new JsonPrimitive(getWaveId()));
    json.add("2", new JsonPrimitive(getWaveletId()));
    {
      JsonElement elem = ((GsonSerializable) getDelta()).toGson(raw, gson);
      json.add("3", elem);
    }
    json.add("4", new JsonPrimitive(getChannelId()));
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      setWaveId(elem.getAsString());
    }
    if (jsonObject.has("2")) {
      JsonElement elem = jsonObject.get("2");
      setWaveletId(elem.getAsString());
    }
    if (jsonObject.has("3")) {
      JsonElement elem = jsonObject.get("3");
      {
        ProtocolWaveletDeltaProtoImpl payload = new ProtocolWaveletDeltaProtoImpl();
        GsonUtil.extractJsonObject(payload, elem, gson, raw);
        setDelta(payload);
      }
    }
    if (jsonObject.has("4")) {
      JsonElement elem = jsonObject.get("4");
      setChannelId(elem.getAsString());
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof SubmitDeltaRequestProtoImpl) {
      return getPB().equals(((SubmitDeltaRequestProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof SubmitDeltaRequest) {
      return SubmitDeltaRequestUtil.isEqual(this, (SubmitDeltaRequest) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}