/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.concurrencycontrol.proto;

import org.waveprotocol.wave.concurrencycontrol.ResponseStatus;
import org.waveprotocol.wave.concurrencycontrol.proto.ResponseStatusProtoImpl;
import org.waveprotocol.wave.concurrencycontrol.WaveletChannelTerminator;
import org.waveprotocol.wave.concurrencycontrol.WaveletChannelTerminatorUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of WaveletChannelTerminator.
 *
 * Generated from org/waveprotocol/wave/concurrencycontrol/clientserver.proto. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class WaveletChannelTerminatorProtoImpl
    implements WaveletChannelTerminator,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.wave.concurrencycontrol.ClientServer.WaveletChannelTerminator> {
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.WaveletChannelTerminator proto = null;
  private org.waveprotocol.wave.concurrencycontrol.ClientServer.WaveletChannelTerminator.Builder protoBuilder = org.waveprotocol.wave.concurrencycontrol.ClientServer.WaveletChannelTerminator.newBuilder();
  public WaveletChannelTerminatorProtoImpl() {
  }

  public WaveletChannelTerminatorProtoImpl(org.waveprotocol.wave.concurrencycontrol.ClientServer.WaveletChannelTerminator proto) {
    this.proto = proto;
  }

  public WaveletChannelTerminatorProtoImpl(WaveletChannelTerminator message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.wave.concurrencycontrol.ClientServer.WaveletChannelTerminator getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.wave.concurrencycontrol.ClientServer.WaveletChannelTerminator proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(WaveletChannelTerminator message) {
    setStatus(new ResponseStatusProtoImpl(message.getStatus()));
  }

  @Override
  public ResponseStatusProtoImpl getStatus() {
    switchToProto();
    return new ResponseStatusProtoImpl(proto.getStatus());
  }

  @Override
  public void setStatus(ResponseStatus value) {
    switchToProtoBuilder();
    protoBuilder.clearStatus();
    protoBuilder.setStatus(getOrCreateResponseStatusProtoImpl(value).getPB());
  }

  /** Get or create a ResponseStatusProtoImpl from a ResponseStatus. */
  private ResponseStatusProtoImpl getOrCreateResponseStatusProtoImpl(ResponseStatus message) {
    if (message instanceof ResponseStatusProtoImpl) {
      return (ResponseStatusProtoImpl) message;
    } else {
      ResponseStatusProtoImpl messageImpl = new ResponseStatusProtoImpl();
      messageImpl.copyFrom(message);
      return messageImpl;
    }
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.wave.concurrencycontrol.ClientServer.WaveletChannelTerminator.newBuilder()
          : org.waveprotocol.wave.concurrencycontrol.ClientServer.WaveletChannelTerminator.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.wave.concurrencycontrol.ClientServer.WaveletChannelTerminator.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    {
      JsonElement elem = ((GsonSerializable) getStatus()).toGson(raw, gson);
      json.add("1", elem);
    }
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      {
        ResponseStatusProtoImpl payload = new ResponseStatusProtoImpl();
        GsonUtil.extractJsonObject(payload, elem, gson, raw);
        setStatus(payload);
      }
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof WaveletChannelTerminatorProtoImpl) {
      return getPB().equals(((WaveletChannelTerminatorProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof WaveletChannelTerminator) {
      return WaveletChannelTerminatorUtil.isEqual(this, (WaveletChannelTerminator) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}