/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.diff;

import org.waveprotocol.wave.federation.ProtocolDocumentOperation;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for DocumentDiffSnapshot.
 *
 * Generated from org/waveprotocol/wave/diff/diff.proto. Do not edit.
 */
public interface DocumentDiffSnapshot {

  /** Does a deep copy from model. */
  void copyFrom(DocumentDiffSnapshot model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns documentId, or null if hasn't been set. */
  String getDocumentId();

  /** Sets documentId. */
  void setDocumentId(String documentId);

  /** Returns whether state has been set. */
  boolean hasState();

  /** Clears the value of state. */
  void clearState();

  /** Returns state, or null if hasn't been set. */
  ProtocolDocumentOperation getState();

  /** Sets state. */
  void setState(ProtocolDocumentOperation state);

  /** Returns whether diff has been set. */
  boolean hasDiff();

  /** Clears the value of diff. */
  void clearDiff();

  /** Returns diff, or null if hasn't been set. */
  ProtocolDocumentOperation getDiff();

  /** Sets diff. */
  void setDiff(ProtocolDocumentOperation diff);

  /** Returns author, or null if hasn't been set. */
  String getAuthor();

  /** Sets author. */
  void setAuthor(String author);

  /** Returns contributor, or null if hasn't been set. */
  List<String> getContributor();

  /** Adds an element to contributor. */
  void addContributor(String value);

  /** Adds a list of elements to contributor. */
  void addAllContributor(List<String> contributor);

  /** Returns the nth element of contributor. */
  String getContributor(int n);

  /** Sets the nth element of contributor. */
  void setContributor(int n, String value);

  /** Returns the length of contributor. */
  int getContributorSize();

  /** Clears contributor. */
  void clearContributor();

  /** Returns addedContributor, or null if hasn't been set. */
  List<String> getAddedContributor();

  /** Adds an element to addedContributor. */
  void addAddedContributor(String value);

  /** Adds a list of elements to addedContributor. */
  void addAllAddedContributor(List<String> addedContributor);

  /** Returns the nth element of addedContributor. */
  String getAddedContributor(int n);

  /** Sets the nth element of addedContributor. */
  void setAddedContributor(int n, String value);

  /** Returns the length of addedContributor. */
  int getAddedContributorSize();

  /** Clears addedContributor. */
  void clearAddedContributor();

  /** Returns removedContributor, or null if hasn't been set. */
  List<String> getRemovedContributor();

  /** Adds an element to removedContributor. */
  void addRemovedContributor(String value);

  /** Adds a list of elements to removedContributor. */
  void addAllRemovedContributor(List<String> removedContributor);

  /** Returns the nth element of removedContributor. */
  String getRemovedContributor(int n);

  /** Sets the nth element of removedContributor. */
  void setRemovedContributor(int n, String value);

  /** Returns the length of removedContributor. */
  int getRemovedContributorSize();

  /** Clears removedContributor. */
  void clearRemovedContributor();

  /** Returns lastModifiedVersion, or null if hasn't been set. */
  double getLastModifiedVersion();

  /** Sets lastModifiedVersion. */
  void setLastModifiedVersion(double lastModifiedVersion);

  /** Returns lastModifiedTime, or null if hasn't been set. */
  double getLastModifiedTime();

  /** Sets lastModifiedTime. */
  void setLastModifiedTime(double lastModifiedTime);
}