/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.diff;

import org.waveprotocol.wave.diff.FetchDiffResponse.*;
import org.waveprotocol.wave.concurrencycontrol.ResponseStatus;
import org.waveprotocol.wave.diff.FetchDiffResponse.WaveletDiff;
import org.waveprotocol.wave.diff.WaveletDiffSnapshot;
import org.waveprotocol.wave.concurrencycontrol.ResponseStatusUtil;
import org.waveprotocol.wave.diff.FetchDiffResponseUtil.WaveletDiffUtil;
import org.waveprotocol.wave.diff.WaveletDiffSnapshotUtil;
import org.waveprotocol.wave.communication.Blob;
import java.util.Iterator;
import java.util.List;

/**
 * Compares {@link FetchDiffResponse}s for equality.
 *
 * Generated from org/waveprotocol/wave/diff/diff.proto. Do not edit.
 */
public final class FetchDiffResponseUtil {

  public static final class WaveletDiffUtil {
    private WaveletDiffUtil() {
    }

    /** Returns true if m1 and m2 are structurally equal. */
    public static boolean isEqual(WaveletDiff m1, WaveletDiff m2) {
      if (!m1.getWaveletId().equals(m2.getWaveletId())) return false;
      if (m1.hasSnapshot() != m2.hasSnapshot()) return false;
      if (m1.hasSnapshot() && !WaveletDiffSnapshotUtil.isEqual(m1.getSnapshot(), m2.getSnapshot())) return false;
      return true;
    }

    /** Returns true if m1 and m2 are equal according to isEqual. */
    public static boolean areAllEqual(List<? extends WaveletDiff> m1,
    List<? extends WaveletDiff> m2) {
      if (m1.size() != m2.size()) return false;
      Iterator<? extends WaveletDiff> i1 = m1.iterator();
      Iterator<? extends WaveletDiff> i2 = m2.iterator();
      while (i1.hasNext()) {
        if (!isEqual(i1.next(), i2.next())) return false;
      }
      return true;
    }

    /** Returns a structural hash code of message. */
    public static int getHashCode(WaveletDiff message) {
      int result = 1;
      result = (31 * result) + message.getWaveletId().hashCode();
      result = (31 * result) + (message.hasSnapshot() ? message.getSnapshot().hashCode() : 0);
      return result;
    }

  }

  private FetchDiffResponseUtil() {
  }

  /** Returns true if m1 and m2 are structurally equal. */
  public static boolean isEqual(FetchDiffResponse m1, FetchDiffResponse m2) {
    if (!ResponseStatusUtil.isEqual(m1.getStatus(), m2.getStatus())) return false;
    if (!WaveletDiffUtil.areAllEqual(m1.getWavelet(), m2.getWavelet())) return false;
    return true;
  }

  /** Returns true if m1 and m2 are equal according to isEqual. */
  public static boolean areAllEqual(List<? extends FetchDiffResponse> m1,
  List<? extends FetchDiffResponse> m2) {
    if (m1.size() != m2.size()) return false;
    Iterator<? extends FetchDiffResponse> i1 = m1.iterator();
    Iterator<? extends FetchDiffResponse> i2 = m2.iterator();
    while (i1.hasNext()) {
      if (!isEqual(i1.next(), i2.next())) return false;
    }
    return true;
  }

  /** Returns a structural hash code of message. */
  public static int getHashCode(FetchDiffResponse message) {
    int result = 1;
    result = (31 * result) + message.getStatus().hashCode();
    result = (31 * result) + message.getWavelet().hashCode();
    return result;
  }

}