/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.federation;

import org.waveprotocol.wave.federation.ProtocolDocumentOperation.*;
import org.waveprotocol.wave.federation.ProtocolDocumentOperation.Component;
import org.waveprotocol.wave.federation.ProtocolDocumentOperation.Component.AnnotationBoundary;
import org.waveprotocol.wave.federation.ProtocolDocumentOperation.Component.ElementStart;
import org.waveprotocol.wave.federation.ProtocolDocumentOperation.Component.ReplaceAttributes;
import org.waveprotocol.wave.federation.ProtocolDocumentOperation.Component.UpdateAttributes;
import org.waveprotocol.wave.federation.ProtocolDocumentOperation.Component.KeyValuePair;
import org.waveprotocol.wave.federation.ProtocolDocumentOperation.Component.KeyValueUpdate;
import org.waveprotocol.wave.federation.ProtocolDocumentOperationUtil.ComponentUtil;
import org.waveprotocol.wave.federation.ProtocolDocumentOperationUtil.ComponentUtil.AnnotationBoundaryUtil;
import org.waveprotocol.wave.federation.ProtocolDocumentOperationUtil.ComponentUtil.ElementStartUtil;
import org.waveprotocol.wave.federation.ProtocolDocumentOperationUtil.ComponentUtil.ReplaceAttributesUtil;
import org.waveprotocol.wave.federation.ProtocolDocumentOperationUtil.ComponentUtil.UpdateAttributesUtil;
import org.waveprotocol.wave.federation.ProtocolDocumentOperationUtil.ComponentUtil.KeyValuePairUtil;
import org.waveprotocol.wave.federation.ProtocolDocumentOperationUtil.ComponentUtil.KeyValueUpdateUtil;
import org.waveprotocol.wave.communication.Blob;
import java.util.Iterator;
import java.util.List;

/**
 * Compares {@link ProtocolDocumentOperation}s for equality.
 *
 * Generated from org/waveprotocol/wave/federation/federation.protodevel. Do not edit.
 */
public final class ProtocolDocumentOperationUtil {

  public static final class ComponentUtil {

    public static final class KeyValuePairUtil {
      private KeyValuePairUtil() {
      }

      /** Returns true if m1 and m2 are structurally equal. */
      public static boolean isEqual(KeyValuePair m1, KeyValuePair m2) {
        if (!m1.getKey().equals(m2.getKey())) return false;
        if (!m1.getValue().equals(m2.getValue())) return false;
        return true;
      }

      /** Returns true if m1 and m2 are equal according to isEqual. */
      public static boolean areAllEqual(List<? extends KeyValuePair> m1,
      List<? extends KeyValuePair> m2) {
        if (m1.size() != m2.size()) return false;
        Iterator<? extends KeyValuePair> i1 = m1.iterator();
        Iterator<? extends KeyValuePair> i2 = m2.iterator();
        while (i1.hasNext()) {
          if (!isEqual(i1.next(), i2.next())) return false;
        }
        return true;
      }

      /** Returns a structural hash code of message. */
      public static int getHashCode(KeyValuePair message) {
        int result = 1;
        result = (31 * result) + message.getKey().hashCode();
        result = (31 * result) + message.getValue().hashCode();
        return result;
      }

    }

    public static final class KeyValueUpdateUtil {
      private KeyValueUpdateUtil() {
      }

      /** Returns true if m1 and m2 are structurally equal. */
      public static boolean isEqual(KeyValueUpdate m1, KeyValueUpdate m2) {
        if (!m1.getKey().equals(m2.getKey())) return false;
        if (m1.hasOldValue() != m2.hasOldValue()) return false;
        if (m1.hasOldValue() && !m1.getOldValue().equals(m2.getOldValue())) return false;
        if (m1.hasNewValue() != m2.hasNewValue()) return false;
        if (m1.hasNewValue() && !m1.getNewValue().equals(m2.getNewValue())) return false;
        return true;
      }

      /** Returns true if m1 and m2 are equal according to isEqual. */
      public static boolean areAllEqual(List<? extends KeyValueUpdate> m1,
      List<? extends KeyValueUpdate> m2) {
        if (m1.size() != m2.size()) return false;
        Iterator<? extends KeyValueUpdate> i1 = m1.iterator();
        Iterator<? extends KeyValueUpdate> i2 = m2.iterator();
        while (i1.hasNext()) {
          if (!isEqual(i1.next(), i2.next())) return false;
        }
        return true;
      }

      /** Returns a structural hash code of message. */
      public static int getHashCode(KeyValueUpdate message) {
        int result = 1;
        result = (31 * result) + message.getKey().hashCode();
        result = (31 * result) + (message.hasOldValue() ? message.getOldValue().hashCode() : 0);
        result = (31 * result) + (message.hasNewValue() ? message.getNewValue().hashCode() : 0);
        return result;
      }

    }

    public static final class ElementStartUtil {
      private ElementStartUtil() {
      }

      /** Returns true if m1 and m2 are structurally equal. */
      public static boolean isEqual(ElementStart m1, ElementStart m2) {
        if (!m1.getType().equals(m2.getType())) return false;
        if (!KeyValuePairUtil.areAllEqual(m1.getAttribute(), m2.getAttribute())) return false;
        return true;
      }

      /** Returns true if m1 and m2 are equal according to isEqual. */
      public static boolean areAllEqual(List<? extends ElementStart> m1,
      List<? extends ElementStart> m2) {
        if (m1.size() != m2.size()) return false;
        Iterator<? extends ElementStart> i1 = m1.iterator();
        Iterator<? extends ElementStart> i2 = m2.iterator();
        while (i1.hasNext()) {
          if (!isEqual(i1.next(), i2.next())) return false;
        }
        return true;
      }

      /** Returns a structural hash code of message. */
      public static int getHashCode(ElementStart message) {
        int result = 1;
        result = (31 * result) + message.getType().hashCode();
        result = (31 * result) + message.getAttribute().hashCode();
        return result;
      }

    }

    public static final class ReplaceAttributesUtil {
      private ReplaceAttributesUtil() {
      }

      /** Returns true if m1 and m2 are structurally equal. */
      public static boolean isEqual(ReplaceAttributes m1, ReplaceAttributes m2) {
        if (m1.hasEmpty() != m2.hasEmpty()) return false;
        if (m1.hasEmpty() && (m1.getEmpty() != m2.getEmpty())) return false;
        if (!KeyValuePairUtil.areAllEqual(m1.getOldAttribute(), m2.getOldAttribute())) return false;
        if (!KeyValuePairUtil.areAllEqual(m1.getNewAttribute(), m2.getNewAttribute())) return false;
        return true;
      }

      /** Returns true if m1 and m2 are equal according to isEqual. */
      public static boolean areAllEqual(List<? extends ReplaceAttributes> m1,
      List<? extends ReplaceAttributes> m2) {
        if (m1.size() != m2.size()) return false;
        Iterator<? extends ReplaceAttributes> i1 = m1.iterator();
        Iterator<? extends ReplaceAttributes> i2 = m2.iterator();
        while (i1.hasNext()) {
          if (!isEqual(i1.next(), i2.next())) return false;
        }
        return true;
      }

      /** Returns a structural hash code of message. */
      public static int getHashCode(ReplaceAttributes message) {
        int result = 1;
        result = (31 * result) + (message.hasEmpty() ? Boolean.valueOf(message.getEmpty()).hashCode() : 0);
        result = (31 * result) + message.getOldAttribute().hashCode();
        result = (31 * result) + message.getNewAttribute().hashCode();
        return result;
      }

    }

    public static final class UpdateAttributesUtil {
      private UpdateAttributesUtil() {
      }

      /** Returns true if m1 and m2 are structurally equal. */
      public static boolean isEqual(UpdateAttributes m1, UpdateAttributes m2) {
        if (m1.hasEmpty() != m2.hasEmpty()) return false;
        if (m1.hasEmpty() && (m1.getEmpty() != m2.getEmpty())) return false;
        if (!KeyValueUpdateUtil.areAllEqual(m1.getAttributeUpdate(), m2.getAttributeUpdate())) return false;
        return true;
      }

      /** Returns true if m1 and m2 are equal according to isEqual. */
      public static boolean areAllEqual(List<? extends UpdateAttributes> m1,
      List<? extends UpdateAttributes> m2) {
        if (m1.size() != m2.size()) return false;
        Iterator<? extends UpdateAttributes> i1 = m1.iterator();
        Iterator<? extends UpdateAttributes> i2 = m2.iterator();
        while (i1.hasNext()) {
          if (!isEqual(i1.next(), i2.next())) return false;
        }
        return true;
      }

      /** Returns a structural hash code of message. */
      public static int getHashCode(UpdateAttributes message) {
        int result = 1;
        result = (31 * result) + (message.hasEmpty() ? Boolean.valueOf(message.getEmpty()).hashCode() : 0);
        result = (31 * result) + message.getAttributeUpdate().hashCode();
        return result;
      }

    }

    public static final class AnnotationBoundaryUtil {
      private AnnotationBoundaryUtil() {
      }

      /** Returns true if m1 and m2 are structurally equal. */
      public static boolean isEqual(AnnotationBoundary m1, AnnotationBoundary m2) {
        if (m1.hasEmpty() != m2.hasEmpty()) return false;
        if (m1.hasEmpty() && (m1.getEmpty() != m2.getEmpty())) return false;
        if (!m1.getEnd().equals(m2.getEnd())) return false;
        if (!KeyValueUpdateUtil.areAllEqual(m1.getChange(), m2.getChange())) return false;
        return true;
      }

      /** Returns true if m1 and m2 are equal according to isEqual. */
      public static boolean areAllEqual(List<? extends AnnotationBoundary> m1,
      List<? extends AnnotationBoundary> m2) {
        if (m1.size() != m2.size()) return false;
        Iterator<? extends AnnotationBoundary> i1 = m1.iterator();
        Iterator<? extends AnnotationBoundary> i2 = m2.iterator();
        while (i1.hasNext()) {
          if (!isEqual(i1.next(), i2.next())) return false;
        }
        return true;
      }

      /** Returns a structural hash code of message. */
      public static int getHashCode(AnnotationBoundary message) {
        int result = 1;
        result = (31 * result) + (message.hasEmpty() ? Boolean.valueOf(message.getEmpty()).hashCode() : 0);
        result = (31 * result) + message.getEnd().hashCode();
        result = (31 * result) + message.getChange().hashCode();
        return result;
      }

    }

    private ComponentUtil() {
    }

    /** Returns true if m1 and m2 are structurally equal. */
    public static boolean isEqual(Component m1, Component m2) {
      if (m1.hasAnnotationBoundary() != m2.hasAnnotationBoundary()) return false;
      if (m1.hasAnnotationBoundary() && !AnnotationBoundaryUtil.isEqual(m1.getAnnotationBoundary(), m2.getAnnotationBoundary())) return false;
      if (m1.hasCharacters() != m2.hasCharacters()) return false;
      if (m1.hasCharacters() && !m1.getCharacters().equals(m2.getCharacters())) return false;
      if (m1.hasElementStart() != m2.hasElementStart()) return false;
      if (m1.hasElementStart() && !ElementStartUtil.isEqual(m1.getElementStart(), m2.getElementStart())) return false;
      if (m1.hasElementEnd() != m2.hasElementEnd()) return false;
      if (m1.hasElementEnd() && (m1.getElementEnd() != m2.getElementEnd())) return false;
      if (m1.hasRetainItemCount() != m2.hasRetainItemCount()) return false;
      if (m1.hasRetainItemCount() && (m1.getRetainItemCount() != m2.getRetainItemCount())) return false;
      if (m1.hasDeleteCharacters() != m2.hasDeleteCharacters()) return false;
      if (m1.hasDeleteCharacters() && !m1.getDeleteCharacters().equals(m2.getDeleteCharacters())) return false;
      if (m1.hasDeleteElementStart() != m2.hasDeleteElementStart()) return false;
      if (m1.hasDeleteElementStart() && !ElementStartUtil.isEqual(m1.getDeleteElementStart(), m2.getDeleteElementStart())) return false;
      if (m1.hasDeleteElementEnd() != m2.hasDeleteElementEnd()) return false;
      if (m1.hasDeleteElementEnd() && (m1.getDeleteElementEnd() != m2.getDeleteElementEnd())) return false;
      if (m1.hasReplaceAttributes() != m2.hasReplaceAttributes()) return false;
      if (m1.hasReplaceAttributes() && !ReplaceAttributesUtil.isEqual(m1.getReplaceAttributes(), m2.getReplaceAttributes())) return false;
      if (m1.hasUpdateAttributes() != m2.hasUpdateAttributes()) return false;
      if (m1.hasUpdateAttributes() && !UpdateAttributesUtil.isEqual(m1.getUpdateAttributes(), m2.getUpdateAttributes())) return false;
      return true;
    }

    /** Returns true if m1 and m2 are equal according to isEqual. */
    public static boolean areAllEqual(List<? extends Component> m1,
    List<? extends Component> m2) {
      if (m1.size() != m2.size()) return false;
      Iterator<? extends Component> i1 = m1.iterator();
      Iterator<? extends Component> i2 = m2.iterator();
      while (i1.hasNext()) {
        if (!isEqual(i1.next(), i2.next())) return false;
      }
      return true;
    }

    /** Returns a structural hash code of message. */
    public static int getHashCode(Component message) {
      int result = 1;
      result = (31 * result) + (message.hasAnnotationBoundary() ? message.getAnnotationBoundary().hashCode() : 0);
      result = (31 * result) + (message.hasCharacters() ? message.getCharacters().hashCode() : 0);
      result = (31 * result) + (message.hasElementStart() ? message.getElementStart().hashCode() : 0);
      result = (31 * result) + (message.hasElementEnd() ? Boolean.valueOf(message.getElementEnd()).hashCode() : 0);
      result = (31 * result) + (message.hasRetainItemCount() ? Integer.valueOf(message.getRetainItemCount()).hashCode() : 0);
      result = (31 * result) + (message.hasDeleteCharacters() ? message.getDeleteCharacters().hashCode() : 0);
      result = (31 * result) + (message.hasDeleteElementStart() ? message.getDeleteElementStart().hashCode() : 0);
      result = (31 * result) + (message.hasDeleteElementEnd() ? Boolean.valueOf(message.getDeleteElementEnd()).hashCode() : 0);
      result = (31 * result) + (message.hasReplaceAttributes() ? message.getReplaceAttributes().hashCode() : 0);
      result = (31 * result) + (message.hasUpdateAttributes() ? message.getUpdateAttributes().hashCode() : 0);
      return result;
    }

  }

  private ProtocolDocumentOperationUtil() {
  }

  /** Returns true if m1 and m2 are structurally equal. */
  public static boolean isEqual(ProtocolDocumentOperation m1, ProtocolDocumentOperation m2) {
    if (!ComponentUtil.areAllEqual(m1.getComponent(), m2.getComponent())) return false;
    return true;
  }

  /** Returns true if m1 and m2 are equal according to isEqual. */
  public static boolean areAllEqual(List<? extends ProtocolDocumentOperation> m1,
  List<? extends ProtocolDocumentOperation> m2) {
    if (m1.size() != m2.size()) return false;
    Iterator<? extends ProtocolDocumentOperation> i1 = m1.iterator();
    Iterator<? extends ProtocolDocumentOperation> i2 = m2.iterator();
    while (i1.hasNext()) {
      if (!isEqual(i1.next(), i2.next())) return false;
    }
    return true;
  }

  /** Returns a structural hash code of message. */
  public static int getHashCode(ProtocolDocumentOperation message) {
    int result = 1;
    result = (31 * result) + message.getComponent().hashCode();
    return result;
  }

}