/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.federation;

import org.waveprotocol.wave.federation.ProtocolHashedVersion;
import org.waveprotocol.wave.federation.ProtocolWaveletOperation;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for ProtocolWaveletDelta.
 *
 * Generated from org/waveprotocol/wave/federation/federation.protodevel. Do not edit.
 */
public interface ProtocolWaveletDelta {

  /** Does a deep copy from model. */
  void copyFrom(ProtocolWaveletDelta model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns hashedVersion, or null if hasn't been set. */
  ProtocolHashedVersion getHashedVersion();

  /** Sets hashedVersion. */
  void setHashedVersion(ProtocolHashedVersion hashedVersion);

  /** Returns author, or null if hasn't been set. */
  String getAuthor();

  /** Sets author. */
  void setAuthor(String author);

  /** Returns operation, or null if hasn't been set. */
  List<? extends ProtocolWaveletOperation> getOperation();

  /** Adds an element to operation. */
  void addOperation(ProtocolWaveletOperation value);

  /** Adds a list of elements to operation. */
  void addAllOperation(List<? extends ProtocolWaveletOperation> operation);

  /** Returns the nth element of operation. */
  ProtocolWaveletOperation getOperation(int n);

  /** Sets the nth element of operation. */
  void setOperation(int n, ProtocolWaveletOperation value);

  /** Returns the length of operation. */
  int getOperationSize();

  /** Clears operation. */
  void clearOperation();

  /** Returns addressPath, or null if hasn't been set. */
  List<String> getAddressPath();

  /** Adds an element to addressPath. */
  void addAddressPath(String value);

  /** Adds a list of elements to addressPath. */
  void addAllAddressPath(List<String> addressPath);

  /** Returns the nth element of addressPath. */
  String getAddressPath(int n);

  /** Sets the nth element of addressPath. */
  void setAddressPath(int n, String value);

  /** Returns the length of addressPath. */
  int getAddressPathSize();

  /** Clears addressPath. */
  void clearAddressPath();
}