/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.federation;

import org.waveprotocol.wave.federation.ProtocolWaveletOperation.MutateDocument;
import org.waveprotocol.wave.federation.ProtocolDocumentOperation;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.ProtoEnums;
import java.util.List;

/**
 * Model interface for ProtocolWaveletOperation.
 *
 * Generated from org/waveprotocol/wave/federation/federation.protodevel. Do not edit.
 */
public interface ProtocolWaveletOperation {

  public interface MutateDocument {

    /** Does a deep copy from model. */
    void copyFrom(MutateDocument model);

    /**
     * Tests if this model is equal to another object.
     * "Equal" is recursively defined as:
     * <ul>
     * <li>both objects implement this interface,</li>
     * <li>all corresponding primitive fields of both objects have the same value, and</li>
     * <li>all corresponding nested-model fields of both objects are "equal".</li>
     * </ul>
     *
     * This is a coarser equivalence than provided by the equals() methods. Two
     * objects may not be equal() to each other, but may be isEqualTo() each other.
     */
    boolean isEqualTo(Object o);

    /** Returns documentId, or null if hasn't been set. */
    String getDocumentId();

    /** Sets documentId. */
    void setDocumentId(String documentId);

    /** Returns documentOperation, or null if hasn't been set. */
    ProtocolDocumentOperation getDocumentOperation();

    /** Sets documentOperation. */
    void setDocumentOperation(ProtocolDocumentOperation documentOperation);
  }

  /** Does a deep copy from model. */
  void copyFrom(ProtocolWaveletOperation model);

  /**
   * Tests if this model is equal to another object.
   * "Equal" is recursively defined as:
   * <ul>
   * <li>both objects implement this interface,</li>
   * <li>all corresponding primitive fields of both objects have the same value, and</li>
   * <li>all corresponding nested-model fields of both objects are "equal".</li>
   * </ul>
   *
   * This is a coarser equivalence than provided by the equals() methods. Two
   * objects may not be equal() to each other, but may be isEqualTo() each other.
   */
  boolean isEqualTo(Object o);

  /** Returns whether addParticipant has been set. */
  boolean hasAddParticipant();

  /** Clears the value of addParticipant. */
  void clearAddParticipant();

  /** Returns addParticipant, or null if hasn't been set. */
  String getAddParticipant();

  /** Sets addParticipant. */
  void setAddParticipant(String addParticipant);

  /** Returns whether removeParticipant has been set. */
  boolean hasRemoveParticipant();

  /** Clears the value of removeParticipant. */
  void clearRemoveParticipant();

  /** Returns removeParticipant, or null if hasn't been set. */
  String getRemoveParticipant();

  /** Sets removeParticipant. */
  void setRemoveParticipant(String removeParticipant);

  /** Returns whether mutateDocument has been set. */
  boolean hasMutateDocument();

  /** Clears the value of mutateDocument. */
  void clearMutateDocument();

  /** Returns mutateDocument, or null if hasn't been set. */
  MutateDocument getMutateDocument();

  /** Sets mutateDocument. */
  void setMutateDocument(MutateDocument mutateDocument);

  /** Returns whether noOp has been set. */
  boolean hasNoOp();

  /** Clears the value of noOp. */
  void clearNoOp();

  /** Returns noOp, or null if hasn't been set. */
  boolean getNoOp();

  /** Sets noOp. */
  void setNoOp(boolean noOp);
}