/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.federation;

import org.waveprotocol.wave.federation.ProtocolWaveletOperation.*;
import org.waveprotocol.wave.federation.ProtocolWaveletOperation.MutateDocument;
import org.waveprotocol.wave.federation.ProtocolDocumentOperation;
import org.waveprotocol.wave.federation.ProtocolWaveletOperationUtil.MutateDocumentUtil;
import org.waveprotocol.wave.federation.ProtocolDocumentOperationUtil;
import org.waveprotocol.wave.communication.Blob;
import java.util.Iterator;
import java.util.List;

/**
 * Compares {@link ProtocolWaveletOperation}s for equality.
 *
 * Generated from org/waveprotocol/wave/federation/federation.protodevel. Do not edit.
 */
public final class ProtocolWaveletOperationUtil {

  public static final class MutateDocumentUtil {
    private MutateDocumentUtil() {
    }

    /** Returns true if m1 and m2 are structurally equal. */
    public static boolean isEqual(MutateDocument m1, MutateDocument m2) {
      if (!m1.getDocumentId().equals(m2.getDocumentId())) return false;
      if (!ProtocolDocumentOperationUtil.isEqual(m1.getDocumentOperation(), m2.getDocumentOperation())) return false;
      return true;
    }

    /** Returns true if m1 and m2 are equal according to isEqual. */
    public static boolean areAllEqual(List<? extends MutateDocument> m1,
    List<? extends MutateDocument> m2) {
      if (m1.size() != m2.size()) return false;
      Iterator<? extends MutateDocument> i1 = m1.iterator();
      Iterator<? extends MutateDocument> i2 = m2.iterator();
      while (i1.hasNext()) {
        if (!isEqual(i1.next(), i2.next())) return false;
      }
      return true;
    }

    /** Returns a structural hash code of message. */
    public static int getHashCode(MutateDocument message) {
      int result = 1;
      result = (31 * result) + message.getDocumentId().hashCode();
      result = (31 * result) + message.getDocumentOperation().hashCode();
      return result;
    }

  }

  private ProtocolWaveletOperationUtil() {
  }

  /** Returns true if m1 and m2 are structurally equal. */
  public static boolean isEqual(ProtocolWaveletOperation m1, ProtocolWaveletOperation m2) {
    if (m1.hasAddParticipant() != m2.hasAddParticipant()) return false;
    if (m1.hasAddParticipant() && !m1.getAddParticipant().equals(m2.getAddParticipant())) return false;
    if (m1.hasRemoveParticipant() != m2.hasRemoveParticipant()) return false;
    if (m1.hasRemoveParticipant() && !m1.getRemoveParticipant().equals(m2.getRemoveParticipant())) return false;
    if (m1.hasMutateDocument() != m2.hasMutateDocument()) return false;
    if (m1.hasMutateDocument() && !MutateDocumentUtil.isEqual(m1.getMutateDocument(), m2.getMutateDocument())) return false;
    if (m1.hasNoOp() != m2.hasNoOp()) return false;
    if (m1.hasNoOp() && (m1.getNoOp() != m2.getNoOp())) return false;
    return true;
  }

  /** Returns true if m1 and m2 are equal according to isEqual. */
  public static boolean areAllEqual(List<? extends ProtocolWaveletOperation> m1,
  List<? extends ProtocolWaveletOperation> m2) {
    if (m1.size() != m2.size()) return false;
    Iterator<? extends ProtocolWaveletOperation> i1 = m1.iterator();
    Iterator<? extends ProtocolWaveletOperation> i2 = m2.iterator();
    while (i1.hasNext()) {
      if (!isEqual(i1.next(), i2.next())) return false;
    }
    return true;
  }

  /** Returns a structural hash code of message. */
  public static int getHashCode(ProtocolWaveletOperation message) {
    int result = 1;
    result = (31 * result) + (message.hasAddParticipant() ? message.getAddParticipant().hashCode() : 0);
    result = (31 * result) + (message.hasRemoveParticipant() ? message.getRemoveParticipant().hashCode() : 0);
    result = (31 * result) + (message.hasMutateDocument() ? message.getMutateDocument().hashCode() : 0);
    result = (31 * result) + (message.hasNoOp() ? Boolean.valueOf(message.getNoOp()).hashCode() : 0);
    return result;
  }

}