/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.federation.proto;

import org.waveprotocol.wave.federation.ProtocolSignature.SignatureAlgorithm;
import org.waveprotocol.wave.federation.ProtocolSignature;
import org.waveprotocol.wave.federation.ProtocolSignatureUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of ProtocolSignature.
 *
 * Generated from org/waveprotocol/wave/federation/federation.protodevel. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class ProtocolSignatureProtoImpl
    implements ProtocolSignature,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.wave.federation.Proto.ProtocolSignature> {
  private org.waveprotocol.wave.federation.Proto.ProtocolSignature proto = null;
  private org.waveprotocol.wave.federation.Proto.ProtocolSignature.Builder protoBuilder = org.waveprotocol.wave.federation.Proto.ProtocolSignature.newBuilder();
  public ProtocolSignatureProtoImpl() {
  }

  public ProtocolSignatureProtoImpl(org.waveprotocol.wave.federation.Proto.ProtocolSignature proto) {
    this.proto = proto;
  }

  public ProtocolSignatureProtoImpl(ProtocolSignature message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.wave.federation.Proto.ProtocolSignature getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.wave.federation.Proto.ProtocolSignature proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(ProtocolSignature message) {
    setSignatureBytes(message.getSignatureBytes());
    setSignerId(message.getSignerId());
    setSignatureAlgorithm(message.getSignatureAlgorithm());
  }

  @Override
  public Blob getSignatureBytes() {
    switchToProto();
    return new Blob(Codec.encode(proto.getSignatureBytes().toByteArray()));
  }

  @Override
  public void setSignatureBytes(Blob value) {
    switchToProtoBuilder();
    protoBuilder.setSignatureBytes(ByteString.copyFrom(Codec.decode(value.getData())));
  }

  @Override
  public Blob getSignerId() {
    switchToProto();
    return new Blob(Codec.encode(proto.getSignerId().toByteArray()));
  }

  @Override
  public void setSignerId(Blob value) {
    switchToProtoBuilder();
    protoBuilder.setSignerId(ByteString.copyFrom(Codec.decode(value.getData())));
  }

  @Override
  public SignatureAlgorithm getSignatureAlgorithm() {
    switchToProto();
    return toPojoEnumSignatureAlgorithm(proto.getSignatureAlgorithm());
  }

  @Override
  public void setSignatureAlgorithm(SignatureAlgorithm value) {
    switchToProtoBuilder();
    protoBuilder.setSignatureAlgorithm(toProtoEnumSignatureAlgorithm(value));
  }

  /** Translates a pojo enum to a proto enum. */
  private org.waveprotocol.wave.federation.Proto.ProtocolSignature.SignatureAlgorithm toProtoEnumSignatureAlgorithm(SignatureAlgorithm value) {
    return org.waveprotocol.wave.federation.Proto.ProtocolSignature.SignatureAlgorithm.valueOf(value.getValue());
  }

  /** Translates a proto enum to a pojo enum. */
  private SignatureAlgorithm toPojoEnumSignatureAlgorithm(org.waveprotocol.wave.federation.Proto.ProtocolSignature.SignatureAlgorithm value) {
    return ProtoEnums.valOf(value.getNumber(), SignatureAlgorithm.values());
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.wave.federation.Proto.ProtocolSignature.newBuilder()
          : org.waveprotocol.wave.federation.Proto.ProtocolSignature.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.wave.federation.Proto.ProtocolSignature.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    json.add("1", new JsonPrimitive(getSignatureBytes().getData()));
    json.add("2", new JsonPrimitive(getSignerId().getData()));
    json.add("3", new JsonPrimitive(getSignatureAlgorithm().getValue()));
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      setSignatureBytes(new Blob(elem.getAsString()));
    }
    if (jsonObject.has("2")) {
      JsonElement elem = jsonObject.get("2");
      setSignerId(new Blob(elem.getAsString()));
    }
    if (jsonObject.has("3")) {
      JsonElement elem = jsonObject.get("3");
      setSignatureAlgorithm(ProtoEnums.valOf(elem.getAsInt(), SignatureAlgorithm.values()));
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof ProtocolSignatureProtoImpl) {
      return getPB().equals(((ProtocolSignatureProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof ProtocolSignature) {
      return ProtocolSignatureUtil.isEqual(this, (ProtocolSignature) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}