/**
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.waveprotocol.wave.federation.proto;

import org.waveprotocol.wave.federation.ProtocolSignature;
import org.waveprotocol.wave.federation.proto.ProtocolSignatureProtoImpl;
import org.waveprotocol.wave.federation.ProtocolSignedDelta;
import org.waveprotocol.wave.federation.ProtocolSignedDeltaUtil;
import org.waveprotocol.wave.communication.Blob;
import org.waveprotocol.wave.communication.Codec;
import org.waveprotocol.wave.communication.ProtoEnums;
import org.waveprotocol.wave.communication.proto.Int52;
import org.waveprotocol.wave.communication.proto.ProtoWrapper;
import org.waveprotocol.wave.communication.gson.GsonException;
import org.waveprotocol.wave.communication.gson.GsonSerializable;
import org.waveprotocol.wave.communication.gson.GsonUtil;
import org.waveprotocol.wave.communication.json.RawStringData;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.protobuf.ByteString;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Server implementation of ProtocolSignedDelta.
 *
 * Generated from org/waveprotocol/wave/federation/federation.protodevel. Do not edit.
 */
// NOTE(kalman): It would be nicer to add a proto serialisation
// utility rather than having this class at all.
public final class ProtocolSignedDeltaProtoImpl
    implements ProtocolSignedDelta,
// Note: fully-qualified path is required for GsonSerializable and ProtoWrapper.
// An import of it is not resolved correctly from inner classes.
// This appears to be a javac bug. The Eclipse compiler handles it fine.
org.waveprotocol.wave.communication.gson.GsonSerializable,
org.waveprotocol.wave.communication.proto.ProtoWrapper<org.waveprotocol.wave.federation.Proto.ProtocolSignedDelta> {
  private org.waveprotocol.wave.federation.Proto.ProtocolSignedDelta proto = null;
  private org.waveprotocol.wave.federation.Proto.ProtocolSignedDelta.Builder protoBuilder = org.waveprotocol.wave.federation.Proto.ProtocolSignedDelta.newBuilder();
  public ProtocolSignedDeltaProtoImpl() {
  }

  public ProtocolSignedDeltaProtoImpl(org.waveprotocol.wave.federation.Proto.ProtocolSignedDelta proto) {
    this.proto = proto;
  }

  public ProtocolSignedDeltaProtoImpl(ProtocolSignedDelta message) {
    copyFrom(message);
  }

  @Override
  public org.waveprotocol.wave.federation.Proto.ProtocolSignedDelta getPB() {
    switchToProto();
    return proto;
  }

  @Override
  public void setPB(org.waveprotocol.wave.federation.Proto.ProtocolSignedDelta proto) {
    this.proto = proto;
    this.protoBuilder = null;
  }

  @Override
  public void copyFrom(ProtocolSignedDelta message) {
    setDelta(message.getDelta());
    clearSignature();
    for (ProtocolSignature field : message.getSignature()) {
      addSignature(new ProtocolSignatureProtoImpl(field));
    }
  }

  @Override
  public Blob getDelta() {
    switchToProto();
    return new Blob(Codec.encode(proto.getDelta().toByteArray()));
  }

  @Override
  public void setDelta(Blob value) {
    switchToProtoBuilder();
    protoBuilder.setDelta(ByteString.copyFrom(Codec.decode(value.getData())));
  }

  @Override
  public List<ProtocolSignatureProtoImpl> getSignature() {
    switchToProto();
    List<ProtocolSignatureProtoImpl> list = new ArrayList<ProtocolSignatureProtoImpl>();
    for (int i = 0; i < getSignatureSize(); i++) {
      ProtocolSignatureProtoImpl message = new ProtocolSignatureProtoImpl(proto.getSignature(i));
      list.add(message);
    }
    return list;
  }

  @Override
  public void addAllSignature(List<? extends ProtocolSignature> values) {
    for (ProtocolSignature message : values) {
      addSignature(message);
    }
  }

  @Override
  public ProtocolSignatureProtoImpl getSignature(int n) {
    switchToProto();
    return new ProtocolSignatureProtoImpl(proto.getSignature(n));
  }

  @Override
  public void setSignature(int n, ProtocolSignature value) {
    switchToProtoBuilder();
    protoBuilder.setSignature(n, getOrCreateProtocolSignatureProtoImpl(value).getPB());
  }

  @Override
  public int getSignatureSize() {
    switchToProto();
    return proto.getSignatureCount();
  }

  @Override
  public void addSignature(ProtocolSignature value) {
    switchToProtoBuilder();
    protoBuilder.addSignature(getOrCreateProtocolSignatureProtoImpl(value).getPB());
  }

  @Override
  public void clearSignature() {
    switchToProtoBuilder();
    protoBuilder.clearSignature();
  }

  /** Get or create a ProtocolSignatureProtoImpl from a ProtocolSignature. */
  private ProtocolSignatureProtoImpl getOrCreateProtocolSignatureProtoImpl(ProtocolSignature message) {
    if (message instanceof ProtocolSignatureProtoImpl) {
      return (ProtocolSignatureProtoImpl) message;
    } else {
      ProtocolSignatureProtoImpl messageImpl = new ProtocolSignatureProtoImpl();
      messageImpl.copyFrom(message);
      return messageImpl;
    }
  }

  private void switchToProto() {
    if (proto == null) {
      proto = protoBuilder.build();
      protoBuilder = null;
    }
  }

  private void switchToProtoBuilder() {
    if (protoBuilder == null) {
      protoBuilder = (proto == null)
          ? org.waveprotocol.wave.federation.Proto.ProtocolSignedDelta.newBuilder()
          : org.waveprotocol.wave.federation.Proto.ProtocolSignedDelta.newBuilder(proto);
      proto = null;
    }
  }

  private void invalidateAll() {
    proto = null;
    protoBuilder = org.waveprotocol.wave.federation.Proto.ProtocolSignedDelta.newBuilder();
  }

  @Override
  public JsonElement toGson(RawStringData raw, Gson gson) {
    JsonObject json = new JsonObject();
    json.add("1", new JsonPrimitive(getDelta().getData()));
    {
      JsonArray array = new JsonArray();
      for (int i = 0; i < getSignatureSize(); i++) {
        JsonElement elem = ((GsonSerializable) getSignature(i)).toGson(raw, gson);
        // NOTE(kalman): if multistage parsing worked, split point would go here.
        array.add(elem);
      }
      json.add("2", array);
    }
    return json;
  }

  @Override
  public void fromGson(JsonElement json, Gson gson, RawStringData raw) throws GsonException {
    JsonObject jsonObject = json.getAsJsonObject();
    // NOTE: always check with has(...) as the json might not have all required
    // fields set; however these (obviously) will need to be set by other means
    // before accessing this object.
    invalidateAll();
    if (jsonObject.has("1")) {
      JsonElement elem = jsonObject.get("1");
      setDelta(new Blob(elem.getAsString()));
    }
    if (jsonObject.has("2")) {
      JsonElement elem = jsonObject.get("2");
      {
        JsonArray array = elem.getAsJsonArray();
        for (int i = 0; i < array.size(); i++) {
          ProtocolSignatureProtoImpl payload = new ProtocolSignatureProtoImpl();
          GsonUtil.extractJsonObject(payload, array.get(i), gson, raw);
          addSignature(payload);
        }
      }
    }
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof ProtocolSignedDeltaProtoImpl) {
      return getPB().equals(((ProtocolSignedDeltaProtoImpl) o).getPB());
    } else {
      return false;
    }
  }

  @Override
  public boolean isEqualTo(Object o) {
    if (equals(o)) {
      return true;
    } else if (o instanceof ProtocolSignedDelta) {
      return ProtocolSignedDeltaUtil.isEqual(this, (ProtocolSignedDelta) o);
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return getPB().hashCode();
  }

  @Override
  public String toString() {
    return getPB().toString();
  }

}